/* * Portable Integrity Protection (PIP) System -
 * Copyright (C) 2015 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Portable Integrity Protection (PIP) System.
 *
 * Portable Integrity Protection (PIP) System is free software: you can
 * redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Portable Integrity Protection (PIP) System is distributed in the hope that it
 * will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Portable Integrity Protection (PIP) System.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#ifndef __PPI_BUFFERMANAGER_H__
#define __PPI_BUFFERMANAGER_H__

#include <stdlib.h>
#include "ppi_config.h"

/* Custom buffer pool for allocating memory mainly for pathnames */

/**************** Config option ******************/

//Configure the number of entries in the pool
#define PPI_BUFFERPOOL_SIZE 200

//Configure to detect overflowing the pool
//#define PPI_BM_BUFFER_OVERFLOW_DETECTION

/*************************************************/

#define PPI_BUFFERPOOL_PER_ENTRY_SIZE (PPI_PATH_WCHAR_SIZE*sizeof(wchar_t))
#define PPI_BUFFERPOOL_ENTRY_SIZE PPI_BUFFERPOOL_PER_ENTRY_SIZE

#define ppi_bm_path_malloc() (wchar_t*)ppi_bm_malloc(PPI_PATH_WCHAR_SIZE*sizeof(wchar_t))

#define ppi_bm_default_malloc() (void *)ppi_bm_path_malloc()
#define ppi_bm_default_mallocSizeInByte PPI_BUFFERPOOL_PER_ENTRY_SIZE
#define ppi_bm_default_mallocSizeInWord PPI_PATH_WCHAR_SIZE


void *ppi_bm_malloc(int size);
void ppi_bm_free(void *buffer);
int ppi_bm_copy(const void *src, void *dst);


/* For detecting buffer overflow */

#ifdef PPI_BM_BUFFER_OVERFLOW_DETECTION

bool ppi_bm_hasCorrupted();
void *ppi_bm_getProtected();

#define PPI_BM_START_DETECT_BUFFER_CORRUPTION bool __bufferCorrupted = ppi_bm_hasCorrupted();
#define PPI_BM_END_DETECT_BUFFER_CORRUPTION if (!__bufferCorrupted && ppi_bm_hasCorrupted()) PPI_CRITICAL("Buffer has been corrupted");

#define PPI_BM_START_DETECT_BUFFER_CORRUPTION_P(x) bool __bufferCorrupted ##x = ppi_bm_hasCorrupted();
#define PPI_BM_END_DETECT_BUFFER_CORRUPTION_P(x) if (!__bufferCorrupted ##x && ppi_bm_hasCorrupted()) PPI_CRITICAL("Buffer has been corrupted");

#else

#define PPI_BM_START_DETECT_BUFFER_CORRUPTION
#define PPI_BM_END_DETECT_BUFFER_CORRUPTION

#define PPI_BM_START_DETECT_BUFFER_CORRUPTION_P(x)
#define PPI_BM_END_DETECT_BUFFER_CORRUPTION_P(x)

#endif

#endif


