/* * Portable Integrity Protection (PIP) System -
 * Copyright (C) 2015 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Portable Integrity Protection (PIP) System.
 *
 * Portable Integrity Protection (PIP) System is free software: you can
 * redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Portable Integrity Protection (PIP) System is distributed in the hope that it
 * will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Portable Integrity Protection (PIP) System.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#ifndef __PPI_CALLS_H__
#define __PPI_CALLS_H__

#ifdef _WIN32_WINNT
#undef _WIN32_WINNT
#define _WIN32_WINNT        0x0501
#endif
#define WIN32
#define NT
#define SECURITY_WIN32

#include <Windows.h>
#include <AccCtrl.h>
#include <Winternl.h>
#include "ppi_utils.h" 
#include "ppi_winternl.h"
#include "ppi_version.h"
#include "ppi_bufferManager.h"

// Use TLS to store whether a call in currently intercepted, or not.
// Once a call has been intercepted, do not intercept again

/* Actual body defined in main.cpp */
void ppi_destoryTls();
struct ppi_tls *ppi_allocateTls();
struct ppi_tls *ppi_getTls();

struct ppi_tls {
	bool ppi_isIntercepted;
};


#define ppi_assign_originalFunc(funcName, library) \
	P_ ##funcName Original ##funcName = (P_ ##funcName)::GetProcAddress(::GetModuleHandle(library), #funcName)


#define EXPAND_ARGC0_(_arg)

#define EXPAND_ARGC1(t1, a1) t1 a1
#define EXPAND_ARGC1_(args_list) EXPAND_ARGC1 args_list

#define EXPAND_ARGC2(t1, a1, ...) t1 a1, \
		EXPAND_ARGC1_((__VA_ARGS__))
#define EXPAND_ARGC2_(args_list) EXPAND_ARGC2 args_list

#define EXPAND_ARGC3(t1, a1, ...) t1 a1, \
		EXPAND_ARGC2_((__VA_ARGS__))
#define EXPAND_ARGC3_(args_list) EXPAND_ARGC3 args_list

#define EXPAND_ARGC4(t1, a1, ...) t1 a1, \
		EXPAND_ARGC3_((__VA_ARGS__))
#define EXPAND_ARGC4_(args_list) EXPAND_ARGC4 args_list

#define EXPAND_ARGC5(t1, a1, ...) t1 a1, \
		EXPAND_ARGC4_((__VA_ARGS__))
#define EXPAND_ARGC5_(args_list) EXPAND_ARGC5 args_list

#define EXPAND_ARGC6(t1, a1, ...) t1 a1, \
		EXPAND_ARGC5_((__VA_ARGS__))
#define EXPAND_ARGC6_(args_list) EXPAND_ARGC6 args_list

#define EXPAND_ARGC7(t1, a1, ...) t1 a1, \
		EXPAND_ARGC6_((__VA_ARGS__))
#define EXPAND_ARGC7_(args_list) EXPAND_ARGC7 args_list

#define EXPAND_ARGC8(t1, a1, ...) t1 a1, \
		EXPAND_ARGC7_((__VA_ARGS__))
#define EXPAND_ARGC8_(args_list) EXPAND_ARGC8 args_list

#define EXPAND_ARGC9(t1, a1, ...) t1 a1, \
		EXPAND_ARGC8_((__VA_ARGS__))
#define EXPAND_ARGC9_(args_list) EXPAND_ARGC9 args_list

#define EXPAND_ARGC10(t1, a1, ...) t1 a1, \
		EXPAND_ARGC9_((__VA_ARGS__))
#define EXPAND_ARGC10_(args_list) EXPAND_ARGC10 args_list

#define EXPAND_ARGC11(t1, a1, ...) t1 a1, \
		EXPAND_ARGC10_((__VA_ARGS__))
#define EXPAND_ARGC11_(args_list) EXPAND_ARGC11 args_list




#define EXPAND_PARC0_(arg)

#define EXPAND_PARC1(t1, a1) (t1) a1
#define EXPAND_PARC1_(args_list) EXPAND_PARC1 args_list

#define EXPAND_PARC2(t1, a1, ...) (t1) a1, \
		EXPAND_PARC1_((__VA_ARGS__))
#define EXPAND_PARC2_(args_list) EXPAND_PARC2 args_list

#define EXPAND_PARC3(t1, a1, ...) (t1) a1, \
		EXPAND_PARC2_((__VA_ARGS__))
#define EXPAND_PARC3_(args_list) EXPAND_PARC3 args_list

#define EXPAND_PARC4(t1, a1, ...) (t1) a1, \
		EXPAND_PARC3_((__VA_ARGS__))
#define EXPAND_PARC4_(args_list) EXPAND_PARC4 args_list

#define EXPAND_PARC5(t1, a1, ...) (t1) a1, \
		EXPAND_PARC4_((__VA_ARGS__))
#define EXPAND_PARC5_(args_list) EXPAND_PARC5 args_list

#define EXPAND_PARC6(t1, a1, ...) (t1) a1, \
		EXPAND_PARC5_((__VA_ARGS__))
#define EXPAND_PARC6_(args_list) EXPAND_PARC6 args_list

#define EXPAND_PARC7(t1, a1, ...) (t1) a1, \
		EXPAND_PARC6_((__VA_ARGS__))
#define EXPAND_PARC7_(args_list) EXPAND_PARC7 args_list

#define EXPAND_PARC8(t1, a1, ...) (t1) a1, \
		EXPAND_PARC7_((__VA_ARGS__))
#define EXPAND_PARC8_(args_list) EXPAND_PARC8 args_list

#define EXPAND_PARC9(t1, a1, ...) (t1) a1, \
		EXPAND_PARC8_((__VA_ARGS__))
#define EXPAND_PARC9_(args_list) EXPAND_PARC9 args_list

#define EXPAND_PARC10(t1, a1, ...) (t1) a1, \
		EXPAND_PARC9_((__VA_ARGS__))
#define EXPAND_PARC10_(args_list) EXPAND_PARC10 args_list

#define EXPAND_PARC11(t1, a1, ...) (t1) a1, \
		EXPAND_PARC10_((__VA_ARGS__))
#define EXPAND_PARC11_(args_list) EXPAND_PARC11 args_list



#define ppi_winapi_declare(call, retValue, argcount, ...) \
	typedef retValue (WINAPI *P_ ##call) ( EXPAND_ARGC ##argcount ##_((__VA_ARGS__)) ); \
	retValue WINAPI Hooked ##call (EXPAND_ARGC ##argcount ##_((__VA_ARGS__))); \
	extern P_ ##call Original ##call;

#define ppi_nonwinapi_declare(call, retValue, argcount, ...) \
	typedef retValue (*P_ ##call) ( EXPAND_ARGC ##argcount ##_((__VA_ARGS__)) ); \
	retValue Hooked ##call (EXPAND_ARGC ##argcount ##_((__VA_ARGS__))); \
	extern P_ ##call Original ##call;


#define ppi_nonwinapi_setError ppi_winapi_setError

#define ppi_winapi_setError(_error) \
	_lastError = _error, \
	_restoreLastError = true



#ifdef PPI_BM_BUFFER_OVERFLOW_DETECTION

#define ppi_winapi_prologue(call, retValue, argcount, ...) \
	retValue WINAPI Hooked ##call (EXPAND_ARGC ##argcount ##_((__VA_ARGS__))) { \
		struct ppi_tls *tls = ppi_getTls(); \
		bool __bufferCorrupted = ppi_bm_hasCorrupted(); \
		DWORD _lastError = 0; BOOL _restoreLastError = false;\
		if (tls->ppi_isIntercepted) { \
__call_original: \
			return Original ##call (EXPAND_PARC ##argcount ##_((__VA_ARGS__))); \
call_original_and_return: \
			if (!__bufferCorrupted && ppi_bm_hasCorrupted()) PPI_CRITICAL("Buffer has corrupted"); \
			tls->ppi_isIntercepted = false; \
			goto __call_original; \
		} \
		tls->ppi_isIntercepted = true;


#define ppi_winapi_epilogue(call, _rv) \
	if (!__bufferCorrupted && ppi_bm_hasCorrupted()) PPI_CRITICAL("Buffer has been corrupted"); \
	tls->ppi_isIntercepted = false; \
	if (_restoreLastError) SetLastError(_lastError); \
	return _rv; \
	}

#else

#define ppi_winapi_prologue(call, retValue, argcount, ...) \
	retValue WINAPI Hooked ##call (EXPAND_ARGC ##argcount ##_((__VA_ARGS__))) { \
		struct ppi_tls *tls = ppi_getTls(); \
		DWORD _lastError = 0; BOOL _restoreLastError = false;\
		if (tls->ppi_isIntercepted) { \
__call_original: \
			return Original ##call (EXPAND_PARC ##argcount ##_((__VA_ARGS__))); \
call_original_and_return: \
			tls->ppi_isIntercepted = false; \
			goto __call_original; \
		} \
		tls->ppi_isIntercepted = true;


#define ppi_winapi_epilogue(call, _rv) \
	tls->ppi_isIntercepted = false; \
	if (_restoreLastError) SetLastError(_lastError); \
	return _rv; \
	}

#endif


#define ppi_call2Hooked_newTls(_call, _retValue, ...) \
	struct ppi_tls *tls = ppi_getTls(); \
	ppi_call2Hooked(_call, _retValue, __VA_ARGS__)


#define ppi_call2Hooked_noTls(_call, _retValue, ...) \
	_retValue = Hooked ##_call(__VA_ARGS__);


#define ppi_callWithInterception(_call, _retValue, ...) \
	struct ppi_tls *tls = ppi_getTls(); \
	tls->ppi_isIntercepted = false; \
	_retValue = _call(__VA_ARGS__);\
	tls->ppi_isIntercepted = true;



#define ppi_call2Hooked(_call, _retValue, ...) \
	tls->ppi_isIntercepted = false; \
	_retValue = Hooked ##_call(__VA_ARGS__);\
	tls->ppi_isIntercepted = true;


#define ppi_callOriginal(_call, ...) \
	Original ##_call (__VA_ARGS__)




/* Structures required for NtCreateFile but not defined */
typedef struct _OBJECT_ATTRIBUTES *POBJECT_ATTRIBUTES;

typedef struct _IO_STATUS_BLOCK *PIO_STATUS_BLOCK;

ppi_winapi_declare(NtCreateFile, NTSTATUS, 11,
  _Out_     PHANDLE, FileHandle,
  _In_      ACCESS_MASK, DesiredAccess,
  _In_      POBJECT_ATTRIBUTES, ObjectAttributes,
  _Out_     PIO_STATUS_BLOCK, IoStatusBlock,
  _In_opt_  PLARGE_INTEGER, AllocationSize,
  _In_      ULONG, FileAttributes,
  _In_      ULONG, ShareAccess,
  _In_      ULONG, CreateDisposition,
  _In_      ULONG, CreateOptions,
  _In_      PVOID, EaBuffer,
  _In_      ULONG, EaLength
);

ppi_winapi_declare(NtCreateFileNew, NTSTATUS, 11,
  _Out_     PHANDLE, FileHandle,
  _In_      ACCESS_MASK, DesiredAccess,
  _In_      POBJECT_ATTRIBUTES, ObjectAttributes,
  _Out_     PIO_STATUS_BLOCK, IoStatusBlock,
  _In_opt_  PLARGE_INTEGER, AllocationSize,
  _In_      ULONG, FileAttributes,
  _In_      ULONG, ShareAccess,
  _In_      ULONG, CreateDisposition,
  _In_      ULONG, CreateOptions,
  _In_      PVOID, EaBuffer,
  _In_      ULONG, EaLength
);


ppi_winapi_declare(CreateProcessW, BOOL, 10,
       _In_opt_        LPCWSTR, lpApplicationName,
       _Inout_opt_     LPWSTR, lpCommandLine,
       _In_opt_        LPSECURITY_ATTRIBUTES, lpProcessAttributes,
       _In_opt_        LPSECURITY_ATTRIBUTES, lpThreadAttributes,
       _In_    BOOL, bInheritHandles,
       _In_    DWORD, dwCreationFlags,
       _In_opt_        LPVOID, lpEnvironment,
       _In_opt_        LPCWSTR, lpCurrentDirectory,
       _In_    LPSTARTUPINFOW, lpStartupInfo,
       _Out_   LPPROCESS_INFORMATION, lpProcessInformation);

ppi_winapi_declare(CreateProcessA, BOOL, 10,
       _In_opt_        LPCSTR, lpApplicationName,
       _Inout_opt_     LPSTR, lpCommandLine,
       _In_opt_        LPSECURITY_ATTRIBUTES, lpProcessAttributes,
       _In_opt_        LPSECURITY_ATTRIBUTES, lpThreadAttributes,
       _In_    BOOL, bInheritHandles,
       _In_    DWORD, dwCreationFlags,
       _In_opt_        LPVOID, lpEnvironment,
       _In_opt_        LPCSTR, lpCurrentDirectory,
       _In_    LPSTARTUPINFOW, lpStartupInfo,
       _Out_   LPPROCESS_INFORMATION, lpProcessInformation);

ppi_winapi_declare(NtOpenFile, NTSTATUS, 6,
  _Out_  PHANDLE, FileHandle,
  _In_   ACCESS_MASK, DesiredAccess,
  _In_   POBJECT_ATTRIBUTES, ObjectAttributes,
  _Out_  PIO_STATUS_BLOCK, IoStatusBlock,
  _In_   ULONG, ShareAccess,
  _In_   ULONG, OpenOptions
);



ppi_winapi_declare(NtQueryDirectoryFileNew, NTSTATUS, 11,
  _In_      HANDLE, FileHandle,
  _In_opt_  HANDLE, Event,
  _In_opt_  PIO_APC_ROUTINE, ApcRoutine,
  _In_opt_  PVOID, ApcContext,
  _Out_     PIO_STATUS_BLOCK, IoStatusBlock,
  _Out_     PVOID, FileInformation,
  _In_      ULONG, Length,
  _In_      FILE_INFORMATION_CLASS, FileInformationClass,
  _In_      BOOLEAN, ReturnSingleEntry,
  _In_opt_  PUNICODE_STRING, FileName,
  _In_      BOOLEAN, RestartScan
);

ppi_winapi_declare(NtOpenFileNew, NTSTATUS, 6,
  _Out_  PHANDLE, FileHandle,
  _In_   ACCESS_MASK, DesiredAccess,
  _In_   POBJECT_ATTRIBUTES, ObjectAttributes,
  _Out_  PIO_STATUS_BLOCK, IoStatusBlock,
  _In_   ULONG, ShareAccess,
  _In_   ULONG, OpenOptions
);

ppi_winapi_declare(NtCloseNew, NTSTATUS, 1,
  _In_   HANDLE, Handle
);

ppi_winapi_declare(NtClose, NTSTATUS, 1,
  _In_   HANDLE, Handle
);


ppi_winapi_declare(NtOpenDirectoryObject, NTSTATUS, 3,
  _Out_  PHANDLE, DirectoryHandle,
  _In_   ACCESS_MASK, DesiredAccess,
  _In_   POBJECT_ATTRIBUTES, ObjectAttributes
);

ppi_winapi_declare(NtQueryDirectoryFile, NTSTATUS, 11,
  _In_      HANDLE, FileHandle,
  _In_opt_  HANDLE, Event,
  _In_opt_  PIO_APC_ROUTINE, ApcRoutine,
  _In_opt_  PVOID, ApcContext,
  _Out_     PIO_STATUS_BLOCK, IoStatusBlock,
  _Out_     PVOID, FileInformation,
  _In_      ULONG, Length,
  _In_      FILE_INFORMATION_CLASS, FileInformationClass,
  _In_      BOOLEAN, ReturnSingleEntry,
  _In_opt_  PUNICODE_STRING, FileName,
  _In_      BOOLEAN, RestartScan
);

ppi_winapi_declare(NtSetInformationFile, NTSTATUS, 5,
  _In_   HANDLE, FileHandle,
  _Out_  PIO_STATUS_BLOCK, IoStatusBlock,
  _In_   PVOID, FileInformation,
  _In_   ULONG, Length,
  _In_   FILE_INFORMATION_CLASS, FileInformationClass
);

ppi_winapi_declare(NtQueryAttributesFile, NTSTATUS, 2,
  _In_   POBJECT_ATTRIBUTES, ObjectAttributes,
  _Out_  PFILE_BASIC_INFORMATION, FileInformation
);


/* May be needed to let untrusted to see files??? */
ppi_winapi_declare(NtQueryDirectoryFile, NTSTATUS, 11,
  _In_      HANDLE, FileHandle,
  _In_opt_  HANDLE, Event,
  _In_opt_  PIO_APC_ROUTINE, ApcRoutine,
  _In_opt_  PVOID, ApcContext,
  _Out_     PIO_STATUS_BLOCK, IoStatusBlock,
  _Out_     PVOID, FileInformation,
  _In_      ULONG, Length,
  _In_      FILE_INFORMATION_CLASS, FileInformationClass,
  _In_      BOOLEAN, ReturnSingleEntry,
  _In_opt_  PUNICODE_STRING, FileName,
  _In_      BOOLEAN, RestartScan
);

ppi_winapi_declare(CreateProcessWNew, BOOL, 10,
	_In_opt_	LPCWSTR, lpApplicationName,
	_Inout_opt_	LPWSTR, lpCommandLine,
	_In_opt_	LPSECURITY_ATTRIBUTES, lpProcessAttributes,
	_In_opt_	LPSECURITY_ATTRIBUTES, lpThreadAttributes,
	_In_	BOOL, bInheritHandles,
	_In_	DWORD, dwCreationFlags,
	_In_opt_	LPVOID, lpEnvironment,
	_In_opt_	LPCWSTR, lpCurrentDirectory,
	_In_	LPSTARTUPINFOW, lpStartupInfo,
	_Out_	LPPROCESS_INFORMATION, lpProcessInformation);

ppi_winapi_declare(CreateProcessANew, BOOL, 10,
	_In_opt_	LPCSTR, lpApplicationName,
	_Inout_opt_	LPSTR, lpCommandLine,
	_In_opt_	LPSECURITY_ATTRIBUTES, lpProcessAttributes,
	_In_opt_	LPSECURITY_ATTRIBUTES, lpThreadAttributes,
	_In_	BOOL, bInheritHandles,
	_In_	DWORD, dwCreationFlags,
	_In_opt_	LPVOID, lpEnvironment,
	_In_opt_	LPCSTR, lpCurrentDirectory,
	_In_	LPSTARTUPINFOW, lpStartupInfo,
	_Out_	LPPROCESS_INFORMATION, lpProcessInformation);


#ifdef WIN81

ppi_winapi_declare(NtQueryFullAttributesFile, NTSTATUS, 2,
  _In_   POBJECT_ATTRIBUTES, ObjectAttributes,
  _Out_  PFILE_NETWORK_OPEN_INFORMATION, FileInformation
);

#endif

ppi_winapi_declare(NtOpenKey, NTSTATUS, 3,
  _Out_       PHANDLE, KeyHandle,
  _In_        ACCESS_MASK, DesiredAccess,
  _In_        POBJECT_ATTRIBUTES, ObjectAttributes
);

ppi_winapi_declare(NtOpenKeyNew, NTSTATUS, 3,
  _Out_       PHANDLE, KeyHandle,
  _In_        ACCESS_MASK, DesiredAccess,
  _In_        POBJECT_ATTRIBUTES, ObjectAttributes
);

ppi_winapi_declare(NtOpenKeyEx, NTSTATUS, 4,
  _Out_       PHANDLE, KeyHandle,
  _In_        ACCESS_MASK, DesiredAccess,
  _In_        POBJECT_ATTRIBUTES, ObjectAttributes,
  _In_        ULONG, OpenOptions
);

ppi_winapi_declare(NtOpenKeyTransacted, NTSTATUS, 4,
  _Out_  PHANDLE, KeyHandle,
  _In_   ACCESS_MASK, DesiredAccess,
  _In_   POBJECT_ATTRIBUTES, ObjectAttributes,
  _In_   HANDLE, TransactionHandle
);

ppi_winapi_declare(NtOpenKeyTransactedEx, NTSTATUS, 5,
  _Out_  PHANDLE, KeyHandle,
  _In_   ACCESS_MASK, DesiredAccess,
  _In_   POBJECT_ATTRIBUTES, ObjectAttributes,
  _In_   ULONG, OpenOptions,
  _In_   HANDLE, TransactionHandle
);

ppi_winapi_declare(NtCreateKey, NTSTATUS, 7,
  _Out_       PHANDLE, KeyHandle,
  _In_        ACCESS_MASK, DesiredAccess,
  _In_        POBJECT_ATTRIBUTES, ObjectAttributes,
  /*_Reserved_*/  ULONG, TitleIndex,
  _In_opt_    PUNICODE_STRING, Class,
  _In_        ULONG, CreateOptions,
  _Out_opt_   PULONG, Disposition
);

ppi_winapi_declare(NtSetValueKey, NTSTATUS, 6,
  _In_      HANDLE, KeyHandle,
  _In_      PUNICODE_STRING, ValueName,
  _In_opt_  ULONG, TitleIndex,
  _In_      ULONG, Type,
  _In_opt_  PVOID, Data,
  _In_      ULONG, DataSize
);


/*************************************************************/


//CreateFile
ppi_winapi_declare(CreateFileW, HANDLE, 7,
	__in      LPCWSTR, lpFileName,
	__in      DWORD, dwDesiredAccess,
	__in      DWORD, dwShareMode,
	__in_opt  LPSECURITY_ATTRIBUTES, lpSecurityAttributes,
	__in      DWORD, dwCreationDisposition,
	__in      DWORD, dwFlagsAndAttributes,
	__in_opt  HANDLE, hTemplateFile
	);


//MoveFile
ppi_winapi_declare(MoveFileExW, BOOL, 3,
	_In_	LPCWSTR, lpExistingFileName,
	_In_opt_	LPCWSTR, lpNewFileName,
	_In_	DWORD, dwFlags);


#endif /* __PPI_CALLS_H__ */