/* * Portable Integrity Protection (PIP) System -
 * Copyright (C) 2015 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Portable Integrity Protection (PIP) System.
 *
 * Portable Integrity Protection (PIP) System is free software: you can
 * redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Portable Integrity Protection (PIP) System is distributed in the hope that it
 * will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Portable Integrity Protection (PIP) System.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#ifndef __PPI_CONFIG_H__
#define __PPI_CONFIG_H__

//Socket port for connecting to helper process 
#define DEFAULT_PORT "27015"

#define xstr(x) str(x)
#define str(s) L#s


/*************** MODIFY BELOW PER SYSTEM SPECIFIC ****************************/

#define PPI_MULTIP_SYSTEM_SID_PREFIX  L"S-1-5-21-2968750198-2704521508-3360548841-"

#define PPI_BENIGN_USER L"IEUser"
#define PPI_BENIGN_USER_ID 1001

#define PPI_UNTRUSTED_USER L"shadow"
#define PPI_UNTRUSTED_USER_ID 1002
#define PPI_UNTRUSTED_USER_PASSWORD L"untrusted"


#define PPI_BENIGN_SIDSTRING PPI_MULTIP_SYSTEM_SID_PREFIX xstr(PPI_BENIGN_USER_ID)
#define PPI_UNTRUSTED_SIDSTRING PPI_MULTIP_SYSTEM_SID_PREFIX xstr(PPI_UNTRUSTED_USER_ID)


/************** MODIFY ABOVE PER SYSTEM SPECIFIC ****************************/

#define PPI_TWO_PRINCIPAL

#define PPI_EVERYONE_SIDSTRING L"S-1-1-0"
#define PPI_USERS_SIDSTRING L"S-1-5-32-545"

#ifndef PPI_TWO_PRINCIPAL
#define PPI_MULTIP_MICROSOFT_USER L"microsoft"
#define PPI_MULTIP_MICROSOFT_ID 1003
#define PPI_MULTIP_MICROSOFT_PASSWORD L"untrusted"
#define PPI_MULTIP_MICROSOFT_ID_STR xstr(PPI_MULTIP_MICROSOFT_ID)
#define PPI_MULTIP_MICROSOFT_REDIRECT_PREFIX L"\\redirect\\microsoft"
#define PPI_MULTIP_MICROSOFT_SIDSTRING PPI_MULTIP_SYSTEM_SID_PREFIX PPI_MULTIP_MICROSOFT_ID_STR
#endif

#define PPI_MULTIP_BENIGN_SIDSTRING PPI_BENIGN_SIDSTRING
#define PPI_MULTIP_BENIGN_USER PPI_BENIGN_USER
#define PPI_MULTIP_BENIGN_PASSWORD NULL
#define PPI_MULTIP_BENIGN_ID PPI_BENIGN_USER_ID
#define PPI_MULTIP_BENIGN_ID_STR xstr(PPI_MULTIP_BENIGN_ID)
#define PPI_MULTIP_BENIGN_REDIRECT_PREFIX L""

#define PPI_MULTIP_UNTRUSTED_SIDSTRING PPI_UNTRUSTED_SIDSTRING
#define PPI_MULTIP_UNTRUSTED_USER PPI_UNTRUSTED_USER
#define PPI_MULTIP_UNTRUSTED_PASSWORD PPI_UNTRUSTED_USER_PASSWORD
#define PPI_MULTIP_UNTRUSTED_ID PPI_UNTRUSTED_USER_ID
#define PPI_MULTIP_UNTRUSTED_ID_STR xstr(PPI_MULTIP_UNTRUSTED_ID)
#define PPI_MULTIP_UNTRUSTED_REDIRECT_PREFIX L"\\redirect\\shadow"

#ifdef PPI_TWO_PRINCIPAL
#define PPI_MULTIP_ALL_USER_IDs PPI_UNTRUSTED_USER_ID, PPI_BENIGN_USER_ID
#else
#define PPI_MULTIP_ALL_USER_IDs PPI_MULTIP_MICROSOFT_ID, PPI_UNTRUSTED_USER_ID, PPI_BENIGN_USER_ID
#endif

#define MULTIP

struct principal_info {
	const wchar_t *fullsid;
	const wchar_t *sid;
	const wchar_t *username;
	const wchar_t *password;
	int id;
};

extern const wchar_t *untrustedInGroupSID[];

#define PPI_PATH_WCHAR_SIZE 2048

#endif /* __PPI_CONFIG_H__ */
