/* * Portable Integrity Protection (PIP) System -
 * Copyright (C) 2015 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Portable Integrity Protection (PIP) System.
 *
 * Portable Integrity Protection (PIP) System is free software: you can
 * redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Portable Integrity Protection (PIP) System is distributed in the hope that it
 * will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Portable Integrity Protection (PIP) System.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#ifndef __PPI_DEBUG__
#define __PPI_DEBUG__


#include "ppi_config.h"
#include <stdio.h>
#include "ppi_calls.h"
#include <Windows.h>

#include "ppi_bufferManager.h"
#include "ppi_utils.h"

#define ppi_log_format "%ls (%ld) " __FUNCTION__ " (%d)"
#define ppi_log_parameters ppi_utils_getProgramPath(), GetCurrentThreadId(), __LINE__
#define BUFFER_SIZE 1024

#define ppi_log_wformat _L(__FUNCTION__) L"(%d)"


//#include "ppi_localdebug.h"


#define PPI_DEBUG


//#define PPI_BEST_PERFORMANCE
#undef PPI_BEST_PERFORMANCE

extern char ppi_debug_buffer[PPI_BUFFERPOOL_ENTRY_SIZE];

BOOL ppi_isDetailedLogProcess();
BOOL ppi_isNewAPIProcess();


#define PPI_CUSTOM_LOG(PATH, format, ...) \
	do {\
		DWORD lastError = GetLastError(); { \
			HANDLE hFile; \
			char *_debug_buffer = (char *)&ppi_debug_buffer; \
			DWORD dwBytesToWrite = 0; \
			DWORD dwBytesWritten = 0; \
			int retry = 0; \
			do { \
			hFile = OriginalCreateFileW(PATH, \
			FILE_APPEND_DATA,          /* open for writing */ \
			FILE_SHARE_WRITE|FILE_SHARE_READ,                      /* do not share */ \
			NULL,                   /* default security */ \
			OPEN_ALWAYS,             /* create new file only */ \
			FILE_ATTRIBUTE_NORMAL,  /* normal file */ \
			NULL);                  /* no attr. template */ \
			retry++; \
			} while (hFile == INVALID_HANDLE_VALUE && retry < 4); \
			if (hFile == INVALID_HANDLE_VALUE ){ SetLastError(lastError); \
			break;} \
			SetLastError(lastError); \
			dwBytesToWrite = _snprintf_s(_debug_buffer, PPI_BUFFERPOOL_ENTRY_SIZE, PPI_BUFFERPOOL_ENTRY_SIZE, ppi_log_format " " format "\r\n", ppi_log_parameters, __VA_ARGS__); \
			WriteFile( \
			hFile,           /* open file handle */ \
			_debug_buffer,			 /* start of data to write */ \
			dwBytesToWrite,  /* number of bytes to write */ \
			&dwBytesWritten, /* number of bytes that were written */ \
			NULL);           /* no overlapped structure */ \
			ppi_callOriginal(NtClose, hFile); \
			retry = 0; \
			do { \
			hFile = OriginalCreateFileW(PPI_ALL_LOG_PATH, \
			FILE_APPEND_DATA,          /* open for writing */ \
			FILE_SHARE_WRITE|FILE_SHARE_READ,                      /* do not share */ \
			NULL,                   /* default security */ \
			OPEN_ALWAYS,             /* create new file only */ \
			FILE_ATTRIBUTE_NORMAL,  /* normal file */ \
			NULL);                  /* no attr. template */ \
			retry++; \
			} while (hFile == INVALID_HANDLE_VALUE && retry < 4); \
			if (hFile == INVALID_HANDLE_VALUE ){ SetLastError(lastError); \
			/*ppi_bm_free(szFileName); ppi_bm_free(buffer);*/ \
			break;} \
			WriteFile( \
			hFile,           /* open file handle */ \
			_debug_buffer,			 /* start of data to write */ \
			dwBytesToWrite,  /* number of bytes to write */ \
			&dwBytesWritten, /* number of bytes that were written */ \
			NULL);           /* no overlapped structure */ \
			ppi_callOriginal(NtClose, hFile); \
			/*ppi_bm_free(szFileName); ppi_bm_free(buffer); */\
		} \
		SetLastError(lastError); \
	} while (0)


#define PPI_ASSERT1(_x) \
	do { \
	if (!(_x)) PPI_CRITICAL("Assert failed: %s", ## _x); \
	} while (0)



#define PPI_ALL_LOG_DIR L"C:\\log\\"
#define PPI_ALL_LOG_PATH PPI_ALL_LOG_DIR L"all2.txt"

//#define PPI_LOG(format, ...) PPI_CUSTOM_LOG(L"C:\\Documents and Settings\\Ubuntu\\Desktop\\example.txt", format, __VA_ARGS__);
//#define PPI_ERROR(format, ...) PPI_CUSTOM_LOG(PPI_ALL_LOG_DIR L"error2.txt", "ERROR " format, __VA_ARGS__);
//#define PPI_CRITICAL(format, ...) PPI_CUSTOM_LOG(PPI_ALL_LOG_DIR L"critical2.txt", "CRITICAL " format, __VA_ARGS__);


#define PPI_HIGH_VIOLATION(format, ...) \
	do { \
		if (ppi_isDetailedLogProcess()) PPI_CUSTOM_LOG(PPI_ALL_LOG_DIR L"detailed2.txt", "DETAIL " format, __VA_ARGS__); \
		PPI_CUSTOM_LOG(PPI_ALL_LOG_DIR L"violations2.txt", "HIGH " format, __VA_ARGS__); \
	} while (0) 

#define PPI_LOW_VIOLATION(format, ...) \
	do { \
		if (ppi_isDetailedLogProcess()) PPI_CUSTOM_LOG(PPI_ALL_LOG_DIR L"detailed2.txt", "DETAIL " format, __VA_ARGS__); \
		PPI_CUSTOM_LOG(PPI_ALL_LOG_DIR L"violations2.txt", "LOW " format, __VA_ARGS__); \
	} while (0) 

#define PPI_BYPASS_ENFORCEMENT(format, ...) PPI_CUSTOM_LOG(PPI_ALL_LOG_DIR L"bypass2.txt", "BYPASS " format, __VA_ARGS__);
#define PPI_INFO(format, ...) PPI_CUSTOM_LOG(PPI_ALL_LOG_DIR L"info2.txt", "INFO " format, __VA_ARGS__);
//#define PPI_INFO(format, ...)


#define PPI_DETAILED_LOG(format, ...) \
	do { \
		if (ppi_isDetailedLogProcess()) PPI_CUSTOM_LOG(PPI_ALL_LOG_DIR L"detailed2.txt", "DETAIL " format, __VA_ARGS__); \
	} while (0)

#define PPI_NEWAPI_LOG(format, ...) \
	do { \
		if (ppi_isNewAPIProcess()) PPI_CUSTOM_LOG(PPI_ALL_LOG_DIR L"newapi2.txt", "NEWAPI " format, __VA_ARGS__); \
	} while (0)


#define PPI_CRITICAL(format, ...) \
	do { \
		if (ppi_isNewAPIProcess()) PPI_CUSTOM_LOG(PPI_ALL_LOG_DIR L"newapi2.txt", "NEWAPI " format, __VA_ARGS__); \
		if (ppi_isDetailedLogProcess()) PPI_CUSTOM_LOG(PPI_ALL_LOG_DIR L"detailed2.txt", "CRITICAL " format, __VA_ARGS__); \
		PPI_CUSTOM_LOG(PPI_ALL_LOG_DIR L"critical2.txt", "CRITICAL " format, __VA_ARGS__);\
	} while (0)

#define PPI_ERROR(format, ...) \
	do { \
		if (ppi_isNewAPIProcess()) PPI_CUSTOM_LOG(PPI_ALL_LOG_DIR L"newapi2.txt", "NEWAPI " format, __VA_ARGS__); \
		if (ppi_isDetailedLogProcess()) PPI_CUSTOM_LOG(PPI_ALL_LOG_DIR L"detailed2.txt", "ERROR " format, __VA_ARGS__); \
		PPI_CUSTOM_LOG(PPI_ALL_LOG_DIR L"error2.txt", "ERROR " format, __VA_ARGS__);\
	} while (0)


#define PPI_BEST_PERFORMANCE


#ifdef PPI_BEST_PERFORMANCE

#undef PPI_INFO
#undef PPI_DETAILED_LOG
#undef PPI_LOG_PERFORMANCE

#define PPI_INFO
#define PPI_DETAILED_LOG
#define PPI_LOG_PERFORMANCE

#else
//#define PPI_INFO(format, ...) PPI_CUSTOM_LOG(L"C:\\Documents and Settings\\Ubuntu\\Desktop\\info2.txt", "INFO " format, __VA_ARGS__);
//#define PPI_TRACEINT(format, ...) PPI_CUSTOM_LOG(L"C:\\Documents and Settings\\Ubuntu\\Desktop\\traceint2.txt", "TRACEINT " format, __VA_ARGS__);
//#define PPI_LOG_PERFORMENCE(format, ...) PPI_CUSTOM_LOG(L"C:\\Documents and Settings\\Ubuntu\\Desktop\\performance2.txt", "PERFORMANCE " format, __VA_ARGS__);
#endif

//#undef ppi_log
//#define ppi_log


#endif /* __PPI_DEBUG__ */