/* * Portable Integrity Protection (PIP) System -
 * Copyright (C) 2015 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Portable Integrity Protection (PIP) System.
 *
 * Portable Integrity Protection (PIP) System is free software: you can
 * redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Portable Integrity Protection (PIP) System is distributed in the hope that it
 * will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Portable Integrity Protection (PIP) System.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#ifndef __PPI_LEVEL_H__
#define __PPI_LEVEL_H__

/* Manage level related information */

#include "ppi_principal.h"
#include "ppi_redirect.h"
#include <map>


int ppi_getFileIsUntrustedW(LPCWSTR fileName, BOOL *lpbFileIsUntrusted);
int ppi_getHandleIsUntrustedW(HANDLE handle, LPCWSTR lpFileName, LPBOOL lpBHandleIsUntrusted);

int ppi_level_fileExistsAndIsUntrustedW(LPCWSTR lpFileName, BOOL *bFileExists, BOOL *bFileIsUntrusted);
int ppi_level_fileExistsAndIsUntrustedA(LPCSTR lpFileName, BOOL *bFileExists, BOOL *bFileIsUntrusted);

int ppi_level_OAExistsAndIsUntrustedW(OBJECT_ATTRIBUTES *ObjectAttributes, BOOL *bFileExists, BOOL *bFileIsUntrusted);
int ppi_level_getMSFileIsUntrustedW(LPCWSTR lpFileName, BOOL *bFileIsUntrusted);

BOOL ppi_level_downgradeFile(LPWSTR filename);
BOOL ppi_level_downgradeDirectory(LPWSTR filename);
BOOL ppi_level_downgradeHandle(HANDLE handle);


/* Manage the custom overrided level information */
class PPI_OVERRIDE_OWNERSHIP_MANAGER {
public:
	static PPI_OVERRIDE_OWNERSHIP_MANAGER &getInstance() {
		static PPI_OVERRIDE_OWNERSHIP_MANAGER instance;
		static bool initialized = false;
		if (!initialized) {
			instance.initialize();
			initialized = true;
		}
		return instance;
	}
	const PPI_PRINCIPAL *getOverridedOwner(const wchar_t *file) const;

	~PPI_OVERRIDE_OWNERSHIP_MANAGER() {
	}

private:
	PPI_OVERRIDE_OWNERSHIP_MANAGER() {};
	PPI_OVERRIDE_OWNERSHIP_MANAGER(PPI_OVERRIDE_OWNERSHIP_MANAGER const &);
	void PPI_OVERRIDE_OWNERSHIP_MANAGER::assign_benign_owner(const wstring s);
	void operator=(PPI_OVERRIDE_OWNERSHIP_MANAGER const &);
	void initialize();
	map<const wstring, const PPI_PRINCIPAL *> override_ownership;
};


#define PPI_GET_OVERRIDED_OWNERSHIP(_p) (PPI_OVERRIDE_OWNERSHIP_MANAGER::getInstance().getOverridedOwner(_p))


/* Abstraction for resource, e.g., handle */
class PPI_Resource {
public:
	PPI_Resource(HANDLE h, PPI_OA *ppi_oa=NULL);
	PPI_Resource(const wchar_t *path);
	bool isTrustedByPrincipal(const PPI_PRINCIPAL *p=PPI_PRINCIPAL_MANAGER::getInstance().getCurrentPrincipal());
	const PPI_PRINCIPAL *PPI_Resource::getUntrustedOwner(const PPI_PRINCIPAL *p=PPI_PRINCIPAL_MANAGER::getInstance().getCurrentPrincipal());

	~PPI_Resource() {
		if (_pSD != NULL) {
			if (_ppi_bm_managePSD)
				ppi_bm_free(_pSD);
			else
				LocalFree((HLOCAL)_pSD);
		}
		if (path_ != NULL) {
			ppi_bm_free(path_);
		}
	};

private:
	const HANDLE _handle;
	const PPI_OA *_ppi_oa;
	PSECURITY_DESCRIPTOR _pSD;
	PSECURITY_DESCRIPTOR _getPSD();
	bool _ppi_bm_managePSD;
	wchar_t *path_;
};



#endif /* __PPI_LEVEL_H__ */
