//
// Portable Integrity Protection (PIP) System -
// Copyright (C) 2015 Secure Systems Laboratory, Stony Brook University
//
// This file is part of Portable Integrity Protection (PIP) System.
//
// Portable Integrity Protection (PIP) System is free software: you can
// redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Portable Integrity Protection (PIP) System is distributed in the hope that it
// will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Portable Integrity Protection (PIP) System.  If not, see
// <http://www.gnu.org/licenses/>.
//

#include "ppi_packet.h"		
#include "ppi_debug.h"

/* Communication to helper */


BOOL ppi_pkt_sendCorrectly(SOCKET handle, char *buffer, size_t size) {
	BOOL rv = FALSE;
	int sent;
	size_t sentSoFar = 0;
	while (sentSoFar < size) {
		sent = send(handle, buffer + sentSoFar, size - sentSoFar, 0);
		if (sent <= 0) {
			//PPI_ERROR("Failed to recv pkt of specified size: %d, got %d", pkt_size, received);
			PPI_ERROR("Connection is closed: sentSoFar: %d", sentSoFar);
			goto out;
		}
		sentSoFar += sent;
	}
	rv = TRUE;
out:
	return rv;
}



BOOL ppi_pkt_recvCorrectly(SOCKET handle, char *buffer, size_t size) {
	BOOL rv = FALSE;
	int received;
	size_t receivedSoFar = 0;
	while (receivedSoFar < size) {
		received = recv(handle, buffer + receivedSoFar, size - receivedSoFar, 0);
		if (received <= 0) {
			//PPI_ERROR("Failed to recv pkt of specified size: %d, got %d", pkt_size, received);
			PPI_ERROR("Connection is closed: receivedSoFar: %d", receivedSoFar);
			goto out;
		}
		receivedSoFar += received;
	}
	rv = TRUE;
out:
	return rv;
}

PUNICODE_STRING ppi_pkt_extractUnicodeString(struct ppi_pkt_UnicodeString *pkt) {
	PUNICODE_STRING unicodeString;
	if (pkt->isNull)
		return NULL;
	unicodeString = &(pkt->unicodeString);
	pkt->unicodeString.Buffer = pkt->Buffer;
	return unicodeString;
}


