/* * Portable Integrity Protection (PIP) System -
 * Copyright (C) 2015 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Portable Integrity Protection (PIP) System.
 *
 * Portable Integrity Protection (PIP) System is free software: you can
 * redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Portable Integrity Protection (PIP) System is distributed in the hope that it
 * will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Portable Integrity Protection (PIP) System.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#ifndef __PPI_PACKET_H__
#define __PPI_PACKET_H__

/* Define the structure for communication with helper, as well as the packer for the data structure */

/*Need the following, or otherwise, struct may not be packed according to standard*/
#include <ntstatus.h>
#define WIN32_NO_STATUS
#include <winsock2.h>
#include <ws2tcpip.h>

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <AccCtrl.h>
#include <Winternl.h>

#include "ppi_winternl.h"
#include "ppi_version.h"

#include "ppi_helper_handleCacheManager.h"


BOOL ppi_pkt_sendCorrectly(SOCKET handle, char *buffer, size_t size);
BOOL ppi_pkt_recvCorrectly(SOCKET handle, char *buffer, size_t size);

#define SEND_PKT_CORRECT(handle, pkt) ppi_pkt_sendCorrectly((handle), (char *)&(pkt), (pkt).l_size)
#define RECV_PKT_CORRECT(handle, pkt) ppi_pkt_recvCorrectly((handle), (char *)&(pkt), (pkt).l_size)


#define PPI_MAX_PATH2 4096


struct ppi_pkt_header {
	size_t pkt_size;
	int cmd_no;
	DWORD pid;
};

struct ppi_response_pkt_header {
	size_t pkt_size;
	int cmd_no;
	NTSTATUS rv;
};

#define PACKET_HEADER struct ppi_pkt_header header
#define RESPONSE_PACKET_HEADER struct ppi_response_pkt_header header
#define l_size header.pkt_size
#define l_cmdno header.cmd_no
#define l_rv header.rv
//#define l_lastError header.lastError

struct ppi_pkt {
	PACKET_HEADER;
};

#define EXPAND_ARG0

#define EXPAND_ARG1(t1, a1) t1 a1;
#define EXPAND_ARG1_(args_list) EXPAND_ARG1 args_list

#define EXPAND_ARG2(t1, a1, ...) t1 a1; \
		EXPAND_ARG1_((__VA_ARGS__))
#define EXPAND_ARG2_(args_list) EXPAND_ARG2 args_list

#define EXPAND_ARG3(t1, a1, ...) t1 a1; \
		EXPAND_ARG2_((__VA_ARGS__))
#define EXPAND_ARG3_(args_list) EXPAND_ARG3 args_list

#define EXPAND_ARG4(t1, a1, ...) t1 a1; \
		EXPAND_ARG3_((__VA_ARGS__))
#define EXPAND_ARG4_(args_list) EXPAND_ARG4 args_list

#define EXPAND_ARG5(t1, a1, ...) t1 a1; \
		EXPAND_ARG4_((__VA_ARGS__))
#define EXPAND_ARG5_(args_list) EXPAND_ARG5 args_list

#define EXPAND_ARG6(t1, a1, ...) t1 a1; \
		EXPAND_ARG5_((__VA_ARGS__))
#define EXPAND_ARG6_(args_list) EXPAND_ARG6 args_list

#define EXPAND_ARG7(t1, a1, ...) t1 a1; \
		EXPAND_ARG6_((__VA_ARGS__))
#define EXPAND_ARG7_(args_list) EXPAND_ARG7 args_list

#define EXPAND_ARG8(t1, a1, ...) t1 a1; \
		EXPAND_ARG7_((__VA_ARGS__))
#define EXPAND_ARG8_(args_list) EXPAND_ARG8 args_list

#define EXPAND_ARG9(t1, a1, ...) t1 a1; \
		EXPAND_ARG8_((__VA_ARGS__))
#define EXPAND_ARG9_(args_list) EXPAND_ARG9 args_list

#define EXPAND_ARG10(t1, a1, ...) t1 a1; \
		EXPAND_ARG9_((__VA_ARGS__))
#define EXPAND_ARG10_(args_list) EXPAND_ARG10 args_list



#define ppi_pkt_declaration(call, argcount, ...) struct ppi_pkt_ ##call {\
	PACKET_HEADER; \
	EXPAND_ARG ##argcount ##_((__VA_ARGS__)) \
}

#define ppi_pkt_response_declaration(call, argcount, ...) struct ppi_pkt_ ##call ##_response {\
	RESPONSE_PACKET_HEADER; \
	EXPAND_ARG ##argcount ##_((__VA_ARGS__))\
}

#define ppi_pkt_prepare_packets(call, pkt, response) \
	struct ppi_pkt_ ##call pkt = ppi_getPkt(call); \
	struct ppi_pkt_ ##call ##_response response = ppi_getPktResponse(call)


#define ppi_getPktResponse(name) { \
	sizeof(struct ppi_pkt_ ##name ##_response), \
	SYS_ ##name, \
	GetCurrentProcessId() \
}

#define ppi_getPkt(name) { \
	sizeof(struct ppi_pkt_ ##name), \
	SYS_ ##name, \
	GetCurrentProcessId() \
}



typedef enum _PPI_SYSCALL_NUM_TYPE {
	SYS_NtCreateFile = 500,
	SYS_NtOpenDirectoryObject,
	SYS_NtSetInformationFile,
	SYS_NtQueryAttributesFile,
#ifdef WIN81
	SYS_NtQueryFullAttributesFile,
#endif
	SYS_NtOpenKeyEx,
	SYS_NtClose,
	SYS_DupToken,
	SYS_NOTUSED
} PPI_SYSCALL_NUM_TYPE;



#define ppi_pkt_packObjectAttributes(_src_oa_ptr, _req_pkt) \
	do { \
		wcsncpy_s(_req_pkt.ObjectAttributes.ObjectName, _src_oa_ptr->ObjectName->Buffer, _src_oa_ptr->ObjectName->Length); \
		_req_pkt.ObjectAttributes.ObjectNameLength = _src_oa_ptr->ObjectName->Length; \
		_req_pkt.ObjectAttributes.Attributes = _src_oa_ptr->Attributes; \
		_req_pkt.ObjectAttributes.RootDirectory_isNull = (_src_oa_ptr->RootDirectory == NULL); \
		_req_pkt.ObjectAttributes.RootDirectory = _src_oa_ptr->RootDirectory; \
		_req_pkt.ObjectAttributes.SecurityDescriptor_isNull = (_src_oa_ptr->SecurityDescriptor == NULL); \
		_req_pkt.ObjectAttributes.SecurityQoS_isNull = (_src_oa_ptr->SecurityQualityOfService == NULL); \
	} while (0)


#define ppi_pkt_unpackObjectAttributes(_req_pkt, _dst_oa, _dst_unicodeString) \
	do { \
		_dst_unicodeString.Length = _dst_unicodeString.MaximumLength = _req_pkt->ObjectAttributes.ObjectNameLength; \
		_dst_unicodeString.Buffer = _req_pkt->ObjectAttributes.ObjectName; \
		_dst_unicodeString.Buffer[_req_pkt->ObjectAttributes.ObjectNameLength] = L'\0'; \
		HANDLE handle = NULL; \
		if (!_req_pkt->ObjectAttributes.RootDirectory_isNull) \
			handle = ppi_helper_handleCacheManager_getPeerHandle(hPeerProcess, _req_pkt->ObjectAttributes.RootDirectory); \
		if (!_req_pkt->ObjectAttributes.SecurityDescriptor_isNull) \
			PPI_INFO("SecurityDescriptor is not NULL, but currently using NULL"); \
		if (!_req_pkt->ObjectAttributes.SecurityQoS_isNull) \
			PPI_INFO("SecurityQoS is not NULL, but currently using NULL"); \
		InitializeObjectAttributes(&_dst_oa, &_dst_unicodeString, _req_pkt->ObjectAttributes.Attributes, handle, NULL); \
	} while (0)

#define ppi_pkt_unpackObjectAttributesCleanUp(_dst_oa)


struct ppi_pkt_OBJECT_ATTRIBUTES {
	wchar_t	ObjectName[PPI_MAX_PATH2];
	USHORT	ObjectNameLength;
	ULONG	Attributes;
	BOOL	RootDirectory_isNull;
	HANDLE	RootDirectory;
	BOOL	SecurityDescriptor_isNull;
	BOOL	SecurityQoS_isNull;
};


ppi_pkt_declaration(NtCreateFile, 9,
  ACCESS_MASK,	DesiredAccess,
  struct ppi_pkt_OBJECT_ATTRIBUTES,	ObjectAttributes,
  LARGE_INTEGER,	AllocationSize,
  ULONG,	FileAttributes,
  ULONG,	ShareAccess,
  ULONG,	CreateDisposition,
  ULONG,	CreateOptions,
  BOOL,		EaBuffer_isNull,
  ULONG,	EaLength
);

ppi_pkt_response_declaration(NtCreateFile, 2,
	HANDLE, FileHandle,
	IO_STATUS_BLOCK,	IoStatusBlock);


ppi_pkt_declaration(NtOpenDirectoryObject, 2,
  ACCESS_MASK, DesiredAccess,
  struct ppi_pkt_OBJECT_ATTRIBUTES,	ObjectAttributes
);

ppi_pkt_response_declaration(NtOpenDirectoryObject, 1,
	HANDLE, DirectoryHandle);


/* NtSetInformationFile */
struct ppi_pkt_FileRenameInformation {
		BOOLEAN ReplaceIfExists;
		HANDLE RootDirectory;
		ULONG FileNameLength;
		wchar_t FileName[PPI_MAX_PATH2];
	};

union FileInformationUnion {
	struct ppi_pkt_FileRenameInformation fileRenameInformation;
};

ppi_pkt_declaration(NtSetInformationFile, 4,
	HANDLE, FileHandle,
	union FileInformationUnion, FileInformation,
	ULONG, Length,
	FILE_INFORMATION_CLASS, FileInformationClass
);

#define ppi_pkt_packFileInformation(_pkt, _FileInformationContent_ptr, _FileInformationClass) \
	do { \
		if (_FileInformationClass == FileRenameInformation) { \
			_pkt.FileInformationClass = (FILE_INFORMATION_CLASS) FileRenameInformation; \
			_pkt.Length = sizeof(union FileInformationUnion); \
			FILE_RENAME_INFORMATION *fileRenameInformation = (FILE_RENAME_INFORMATION *)_FileInformationContent_ptr; \
			_pkt.FileInformation.fileRenameInformation.ReplaceIfExists = fileRenameInformation->ReplaceIfExists; \
			_pkt.FileInformation.fileRenameInformation.RootDirectory = fileRenameInformation->RootDirectory; \
			_pkt.FileInformation.fileRenameInformation.FileNameLength = fileRenameInformation->FileNameLength; \
			memcpy(_pkt.FileInformation.fileRenameInformation.FileName, &(fileRenameInformation->FileName), fileRenameInformation->FileNameLength); \
			_pkt.FileInformation.fileRenameInformation.FileName[fileRenameInformation->FileNameLength/2] = L'\0'; \
		} \
	} while (0)


ppi_pkt_response_declaration(NtSetInformationFile, 2,
	IO_STATUS_BLOCK, IoStatusBlock,
	BOOL, ppi_RetryWithShadow);


//NtQueryAttributesFile

ppi_pkt_declaration(NtQueryAttributesFile, 1,
  struct ppi_pkt_OBJECT_ATTRIBUTES,	ObjectAttributes
);

ppi_pkt_response_declaration(NtQueryAttributesFile, 1,
	FILE_BASIC_INFORMATION, FileInformation);


/***************** Win 8.1 ******************/
#ifdef WIN81

ppi_pkt_declaration(NtQueryFullAttributesFile, 1,
  struct ppi_pkt_OBJECT_ATTRIBUTES,	ObjectAttributes
);

ppi_pkt_response_declaration(NtQueryFullAttributesFile, 1,
	FILE_NETWORK_OPEN_INFORMATION, FileInformation);

#endif


ppi_pkt_declaration(NtOpenKeyEx, 3,
  ACCESS_MASK, DesiredAccess,
  struct ppi_pkt_OBJECT_ATTRIBUTES,	ObjectAttributes,
  ULONG, OpenOptions
);

ppi_pkt_response_declaration(NtOpenKeyEx, 1,
	HANDLE, KeyHandle
);


ppi_pkt_declaration(NtClose, 1,
	HANDLE, Handle
);

ppi_pkt_response_declaration(NtClose, 1,
	int, notused
);


ppi_pkt_declaration(DupToken, 1,
	int, id
);

ppi_pkt_response_declaration(DupToken, 1,
	HANDLE, token
);





struct ppi_pkt_UnicodeString {
	BOOL isNull;
	UNICODE_STRING unicodeString;
	wchar_t Buffer[PPI_MAX_PATH2];
};

#define ppi_pkt_packUnicodeString(_pktUnicodeString, _unicodeString_ptr) \
	do { \
		if (_unicodeString_ptr == NULL) \
			_pktUnicodeString.isNull = True; \
		else { \
			_pktUnicodeString.isNull = False; \
			_pktUnicodeString.unicodeString.Length = _unicodeString_ptr->Length; \
			_pktUnicodeString.unicodeString.MaximumLength = _unicodeString_ptr->MaximumLength; \
			memcpy(&(_pktUnicodeString.Buffer), _unicodeString_ptr->Buffer, _pktUnicodeString.Length); \
		} \
	} while (0)

PUNICODE_STRING ppi_pkt_extractUnicodeString(struct ppi_pkt_UnicodeString *);


ppi_pkt_declaration(NtCreateKey, 6,
  ACCESS_MASK, DesiredAccess,
  struct ppi_pkt_OBJECT_ATTRIBUTES,	ObjectAttributes,
  ULONG, TitleIndex,
  _In_opt_    PUNICODE_STRING, Class,
  ULONG, CreateOptions,
  _Out_opt_   PULONG, Disposition
);




#endif /* __PPI_PACKET_H__ */

