/* * Portable Integrity Protection (PIP) System -
 * Copyright (C) 2015 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Portable Integrity Protection (PIP) System.
 *
 * Portable Integrity Protection (PIP) System is free software: you can
 * redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Portable Integrity Protection (PIP) System is distributed in the hope that it
 * will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Portable Integrity Protection (PIP) System.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#ifndef __PPI_REDIRECT_H__
#define __PPI_REDIRECT_H__

#include "ppi_utils.h"
#include "ppi_principal.h"
#include <vector>

using namespace std;



extern const wchar_t *ppi_redirectablePath[];



bool ppi_recursivelyCreateDirectoriesIgnoreLastW(const wchar_t *fullPath);

#define ppi_isRedirectableW(_path) ((wcsncmp(_path, L"\\\\?\\", 4) != 0 && ppi_utils_wstringStartWithArray(_path, ppi_redirectablePath)) || \
		((wcsncmp(_path, L"\\\\?\\", 4) == 0 || wcsncmp(_path, L"\\??\\", 4) == 0) && ppi_utils_wstringStartWithArray(_path + 4, ppi_redirectablePath)))



void ppi_deallocShadowPath(wchar_t *shadowPath);




UNICODE_STRING MULTIP *ppi_convertWChart2UnicodeAllos(const wchar_t *path);
void ppi_deallocUnicodeShadowPath(UNICODE_STRING *shadowPath);

OBJECT_ATTRIBUTES *ppi_convertRegObjectAttributes2Benign(OBJECT_ATTRIBUTES *originalOA);
OBJECT_ATTRIBUTES * MULTIP ppi_createObjectAttributesWithPrivatePath(const OBJECT_ATTRIBUTES *originalOA);
void ppi_deallocObjectAttributes(OBJECT_ATTRIBUTES *oa);

UNICODE_STRING MULTIP *ppi_convertUnicode2PrivateAlloc(const UNICODE_STRING *originalPath, const wchar_t *redirectPrefix);
UNICODE_STRING *PPI_replaceUserIDinRegAlloc(const wchar_t *originalReg, const wchar_t *toReplace);



//BOOL MULTIP ppi_OAIsRedirectable(const OBJECT_ATTRIBUTES *oa);
#define ppi_OAIsRedirectable(_oa) ((wcsncmp((_oa->ObjectName->Buffer), L"\\\\?\\", 4) != 0 && ppi_utils_wstringStartWithArray((_oa->ObjectName->Buffer), ppi_redirectablePath)) || \
		((wcsncmp((_oa->ObjectName->Buffer), L"\\\\?\\", 4) == 0 || wcsncmp((_oa->ObjectName->Buffer), L"\\??\\", 4) == 0) && ppi_utils_wstringStartWithArray((_oa->ObjectName->Buffer) + 4, ppi_redirectablePath)))

int ppi_insertRedirectPrefix(const wchar_t *originalPath, const wchar_t *redirectPrefix, wchar_t *buffer, size_t bufferSize);


class PPI_DirectoryHandle {
public:
	PPI_DirectoryHandle(HANDLE handle, const PPI_PRINCIPAL *principal=PPI_PRINCIPAL_MANAGER::getInstance().getCurrentPrincipal());
	const HANDLE handle;
	const wchar_t *getHandlePath() const;
	~PPI_DirectoryHandle();

	const vector<wchar_t *> &getAllDirectoryPath();
private:
	const PPI_PRINCIPAL *_principal;
	wchar_t *handlePath;
	vector<wchar_t *> allDirectoryPaths;
};


class PPI_OA {
public:
	const OBJECT_ATTRIBUTES *original_oa;
	
	PPI_OA(OBJECT_ATTRIBUTES *oa, const PPI_PRINCIPAL *principal=PPI_PRINCIPAL_MANAGER::getInstance().getCurrentPrincipal()): original_oa(oa), _principal(principal){
	
		_privateOA = NULL;
		_firstReadableOA = NULL;

		this->_printablePath = (wchar_t *)ppi_bm_malloc(PPI_PATH_WCHAR_SIZE*sizeof(wchar_t));
		if (this->_printablePath)
			ppi_utils_extractFilePathFromOA(oa, this->_printablePath, PPI_PATH_WCHAR_SIZE*sizeof(wchar_t));

	};
	//PPI_OA(OBJECT_ATTRIBUTES *oa): original_oa(oa), _principal(PPI_PRINCIPAL_MANAGER::getInstance().getCurrentPrincipal()) {};

	bool isRedirectable() const;
	OBJECT_ATTRIBUTES *getPrivateOA();
	OBJECT_ATTRIBUTES *getFirstReadableOA();
	bool isExist() const;
	bool copyFile2private();
	const wchar_t *getPrintablePath() const{ return _printablePath; };

	~PPI_OA() {
		if (_privateOA != NULL) _freeAllocatedOA(_privateOA);
		if ((_firstReadableOA != NULL) && (_firstReadableOA != _privateOA)) _freeAllocatedOA(_firstReadableOA);
		if (_printablePath != NULL) ppi_bm_free(_printablePath);
	};

private:
	wchar_t *_printablePath;
	OBJECT_ATTRIBUTES *_privateOA;
	OBJECT_ATTRIBUTES *_firstReadableOA;
	const PPI_PRINCIPAL *_principal; //list<PPI_PRINCIPAL *> *trustedPrincipals;
	void _freeAllocatedOA(OBJECT_ATTRIBUTES *oa);
};




#endif __PPI_REDIRECT_H__
