/* * Portable Integrity Protection (PIP) System -
 * Copyright (C) 2015 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Portable Integrity Protection (PIP) System.
 *
 * Portable Integrity Protection (PIP) System is free software: you can
 * redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Portable Integrity Protection (PIP) System is distributed in the hope that it
 * will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Portable Integrity Protection (PIP) System.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#ifndef __PPI_UTILS_H__
#define __PPI_UTILS_H__


BOOL ppi_utils_wstringInArray(const wchar_t *toFind, const wchar_t *stringArray[]);

wchar_t *ppi_utils_convertChar2Wchar(const char *input);
void ppi_utils_deleteCharBuffer(wchar_t *buffer);

int ppi_utils_convertChar2Wchar(const char *input, wchar_t *buffer, size_t buflen);

//BOOL ppi_level_getProcessIsUntrusted();

BOOL ppi_utils_wstringStartWithArray(const wchar_t *toFind, const wchar_t *stringArray[]);
int ppi_utils_fileExist(LPCWSTR fileName, BOOL *fileExists);

#define PPI_IS_UNTRUSTED (ppi_level_getProcessIsUntrusted())
#define PPI_IS_BENIGN (!PPI_IS_UNTRUSTED)



void ppi_utils_grantUntrustedReadAccess(LPCWSTR filename);


#define ppi_utils_extractFilePathFromOA(_oa, _wpath_ptr, _pathlenByte) \
	do { \
		int startwByte = 0; \
		int bufferStart = 0; \
		if (_wpath_ptr == NULL) { \
			PPI_ERROR("_wpath_ptr is NULL"); \
			break; \
		} \
		if ((_oa)->ObjectName->Buffer == NULL) { \
			_wpath_ptr[0] = L'\0'; \
			break;\
		} \
		if (wcsncmp((_oa)->ObjectName->Buffer, L"\\??\\", 4) == 0) \
			startwByte += 4; \
		/*else if ((_oa)->RootDirectory != NULL) \
			bufferStart = GetFinalPathNameByHandle((_oa)->RootDirectory, _wpath_ptr, _pathlenByte/2, FILE_NAME_OPENED);*/ \
		int lengthByte = (_oa)->ObjectName->Length - startwByte*sizeof(wchar_t) + bufferStart; \
		if (lengthByte > (_pathlenByte)) { \
			PPI_ERROR("Insufficient space for holding the result!!!, require: %d, given: %d", lengthByte, _pathlenByte); \
			lengthByte = (_pathlenByte); \
		} \
		memcpy(_wpath_ptr + bufferStart, _oa->ObjectName->Buffer+ startwByte, lengthByte); \
		_wpath_ptr[lengthByte/2] = L'\0'; \
	} while (0)



//wchar_t *ppi_wcsnstr(const wchar_t *string, const wchar_t *handle, size_t n);
const wchar_t *ppi_firstUntrustedSID(const wchar_t *string, size_t n);


BOOL ppi_utils_intInArray(int toFind, const int intArray[]);

const wchar_t *ppi_utils_getProgramPath();

#endif