//
// Portable Integrity Protection (PIP) System -
// Copyright (C) 2015 Secure Systems Laboratory, Stony Brook University
//
// This file is part of Portable Integrity Protection (PIP) System.
//
// Portable Integrity Protection (PIP) System is free software: you can
// redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Portable Integrity Protection (PIP) System is distributed in the hope that it
// will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Portable Integrity Protection (PIP) System.  If not, see
// <http://www.gnu.org/licenses/>.
//

#include "ppi_helper.h"
#include "ppi_debug.h"
#include "ppi_packet.h"

#include "ppi_helper_handleCacheManager.h"
#include "ppi_config.h"

#define CASE_USER_PASSWD(_x) case PPI_MULTIP_ ##_x ##_ID: \
	username = PPI_MULTIP_ ##_x ##_USER; \
	password = PPI_MULTIP_ ##_x ##_PASSWORD; \
	break;


ppi_helper_def_call(DupToken) {
	
	HANDLE handle = NULL;
	HANDLE primaryToken;
	HANDLE _token;

	wchar_t *username = NULL;
	wchar_t *password = NULL;


	switch(req->id) {
#ifndef PPI_TWO_PRINCIPAL		
	CASE_USER_PASSWD(MICROSOFT);
#endif
	CASE_USER_PASSWD(UNTRUSTED);
	default:
		response->l_rv = STATUS_ACCESS_DENIED;
		return;
	}

	if (LogonUser(username, L".", password, LOGON32_LOGON_INTERACTIVE, LOGON32_PROVIDER_DEFAULT, &primaryToken)) {
		if (!DuplicateToken(primaryToken, SecurityIdentification, &handle))
			PPI_CRITICAL("Failed to duplicate token with impersonation requirement: error %d", GetLastError());
		else
			_token = handle;
		CloseHandle(primaryToken);
	}

	if (DuplicateHandle(GetCurrentProcess(), _token, hPeerProcess, &response->token, 0, true, DUPLICATE_CLOSE_SOURCE|DUPLICATE_SAME_ACCESS) != 0) {
		PPI_INFO("[+] Duplicate token succeed: handle in peer");
		response->l_rv = STATUS_SUCCESS;
	} else {
		PPI_CRITICAL("Failed to duplicate handle");
		response->l_rv = STATUS_ACCESS_DENIED;
	}
}