//
// Portable Integrity Protection (PIP) System -
// Copyright (C) 2015 Secure Systems Laboratory, Stony Brook University
//
// This file is part of Portable Integrity Protection (PIP) System.
//
// Portable Integrity Protection (PIP) System is free software: you can
// redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Portable Integrity Protection (PIP) System is distributed in the hope that it
// will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Portable Integrity Protection (PIP) System.  If not, see
// <http://www.gnu.org/licenses/>.
//

#include "ppi_helper.h"
#include "ppi_debug.h"
#include "ppi_packet.h"
#include "ppi_calls.h"

#include "ppi_level.h"


ppi_assign_originalFunc(NtCreateFile, L"ntdll");


ppi_helper_def_call(NtCreateFile) {
	
	OBJECT_ATTRIBUTES ObjectAttributes;
	UNICODE_STRING ObjectName;

	ppi_pkt_unpackObjectAttributes(req, ObjectAttributes, ObjectName);

	HANDLE FileHandle = NULL;

	wchar_t *fileName = ObjectName.Buffer;
	if (wcsncmp(fileName, L"\\??\\", 4) == 0)
		fileName += 4;


	BOOL bFileIsUntrusted, bFileExist;
	if (ppi_level_OAExistsAndIsUntrustedW(&ObjectAttributes, &bFileExist, &bFileIsUntrusted)) {
		goto deny_open;
	}

	if ((req->DesiredAccess & (GENERIC_WRITE|DELETE|FILE_APPEND_DATA)) && bFileExist && !bFileIsUntrusted) {
		PPI_LOW_VIOLATION("Denying opening file %ls for writing or deleting", fileName);
		goto deny_open;
		PPI_BYPASS_ENFORCEMENT("Allowing opening benign file %ls for writing or deleting", fileName);
	}

	response->l_rv = OriginalNtCreateFile(&FileHandle, req->DesiredAccess|READ_CONTROL|WRITE_DAC, &ObjectAttributes, &(response->IoStatusBlock), &(req->AllocationSize),
				req->FileAttributes, req->ShareAccess, req->CreateDisposition, req->CreateOptions, NULL, 0);

	//PPI_CRITICAL("rv is %d, success: %d", response->l_rv, STATUS_SUCCESS);
		
	if (response->l_rv != STATUS_SUCCESS) {
		PPI_ERROR("Helper really cannot open file %ls, error status: %Xh", fileName, response->l_rv);
		goto out;
	} else {
		//Downgrade the file that is newly created for untrusted processes
		if (!bFileExist) {
			//if (ppi_level_downgradeFile(fileName)) {
			if (ppi_level_downgradeHandle(FileHandle)) {
				PPI_INFO("File is downgraded: %ls", fileName);
			} else {
				PPI_CRITICAL("Failed to downgrade file %ls", fileName);
			}
		}
	}
	
	HANDLE peerFileHandle;

	if (DuplicateHandle(GetCurrentProcess(), FileHandle, hPeerProcess, &response->FileHandle, req->DesiredAccess|READ_CONTROL, true, DUPLICATE_CLOSE_SOURCE) != 0) {
		PPI_INFO("[+] Duplicate succeed: handle in peer: %Xh for file %ls, Access: %Xh", response->FileHandle, fileName, req->DesiredAccess);
	} else {
		PPI_CRITICAL("Failed to duplicate handle");
		response->l_rv = STATUS_ACCESS_DENIED;
	}

	/* FileHandle is closed by DuplicateHandle */
out:
	ppi_pkt_unpackObjectAttributesCleanUp(ObjectAttributes);
	return;
	
deny_open:
	response->l_rv = STATUS_ACCESS_DENIED;
	goto out;
}