//
// Portable Integrity Protection (PIP) System -
// Copyright (C) 2015 Secure Systems Laboratory, Stony Brook University
//
// This file is part of Portable Integrity Protection (PIP) System.
//
// Portable Integrity Protection (PIP) System is free software: you can
// redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Portable Integrity Protection (PIP) System is distributed in the hope that it
// will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Portable Integrity Protection (PIP) System.  If not, see
// <http://www.gnu.org/licenses/>.
//

#include "ppi_helper.h"
#include "ppi_debug.h"
#include "ppi_packet.h"
#include "ppi_calls.h"

#include "ppi_level.h"


ppi_assign_originalFunc(NtSetInformationFile, L"ntdll");


ppi_helper_def_call(NtSetInformationFile) {
	
	HANDLE localHandle = NULL;
	response->ppi_RetryWithShadow = FALSE;
	
	if (DuplicateHandle(hPeerProcess, req->FileHandle, GetCurrentProcess(), &localHandle, NULL, true, DUPLICATE_SAME_ACCESS) != 0) {
		PPI_INFO("[+] Duplicate for rename succeed, handle in peer: %Xh", req->FileHandle);
	} else {
		PPI_CRITICAL("Failed to duplicate handle, cannot rename file %d!!!", GetLastError());
		response->l_rv = STATUS_ACCESS_DENIED;
		return;
	}

	switch(req->FileInformationClass) {
	case FileRenameInformation: {
		if (req->FileInformation.fileRenameInformation.RootDirectory != NULL) {
			PPI_CRITICAL("RootDirectory Should be NULL");
			req->FileInformation.fileRenameInformation.RootDirectory = NULL;
		}

		//CHECK IF THE FILE IS UNTRUSTED OR NOT BEFORE RENAMING!!!!
		BOOL handleIsUntrusted;
		if (ppi_getHandleIsUntrustedW(localHandle, NULL, &handleIsUntrusted)) {
			PPI_CRITICAL("Failed to get whether handle to be renamed is untrusted or not");
			goto deny_action;
		} else if (!handleIsUntrusted){
			PPI_CRITICAL("Failed to get whether handle to be renamed is untrusted or not");
			goto deny_action;
		}

		//Not allow overwriting benign file
		BOOL bFileExists, bFileIsUntrusted;

		wchar_t *fileName = (wchar_t *)&(req->FileInformation.fileRenameInformation.FileName);


		if (ppi_level_fileExistsAndIsUntrustedW(fileName, &bFileExists, &bFileIsUntrusted)) {
			PPI_ERROR("Failed to check if rename target is untrusted or not, assuming benign %ls and not allow rename", fileName);
			goto deny_action;
		}
		PPI_INFO("target of rename is %ls, exists: %d, isuntrusted: %d", fileName, bFileExists, bFileIsUntrusted);
		if (bFileExists && !bFileIsUntrusted && req->FileInformation.fileRenameInformation.ReplaceIfExists) {		
			PPI_LOW_VIOLATION("Denying renaming to overwrite benign file %ls, suggesting client to retry", fileName);
			response->ppi_RetryWithShadow = TRUE;
			goto deny_action;
			PPI_BYPASS_ENFORCEMENT("Allow renaming to overwrite benign file %ls", fileName);
		}

		response->l_rv = OriginalNtSetInformationFile(localHandle, &response->IoStatusBlock, &req->FileInformation, req->Length, req->FileInformationClass);
		PPI_INFO("Rename to %ls gave %Xh", fileName, response->l_rv);
		break;

	}
	default:
		PPI_CRITICAL("Unhandled SetInformationFile options!");
		goto deny_action;
	}

out:
	if (localHandle != NULL)
		CloseHandle(localHandle);

	return;

deny_action:
	response->l_rv = STATUS_ACCESS_DENIED;
	goto out;

	
}