//
// Portable Integrity Protection (PIP) System -
// Copyright (C) 2015 Secure Systems Laboratory, Stony Brook University
//
// This file is part of Portable Integrity Protection (PIP) System.
//
// Portable Integrity Protection (PIP) System is free software: you can
// redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Portable Integrity Protection (PIP) System is distributed in the hope that it
// will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Portable Integrity Protection (PIP) System.  If not, see
// <http://www.gnu.org/licenses/>.
//

#include "ppi_helper.h"
#include "ppi_debug.h"
#include "ppi_packet.h"
#include "ppi_calls.h"

#include "ppi_level.h"


ppi_assign_originalFunc(NtOpenKeyEx, L"ntdll");

ppi_helper_def_call(NtOpenKeyEx) {
	
	OBJECT_ATTRIBUTES ObjectAttributes;
	UNICODE_STRING ObjectName;
	HANDLE KeyHandle = NULL;

	ppi_pkt_unpackObjectAttributes(req, ObjectAttributes, ObjectName);

	if ((req->DesiredAccess & KEY_WRITE) == 0) {
		PPI_LOW_VIOLATION("Denying opening reg %ls for writing", req->ObjectAttributes.ObjectName);
		goto deny_open;
		PPI_BYPASS_ENFORCEMENT("Allowing opening reg %ls for writing", req->ObjectAttributes.ObjectName);
	}
	
	response->l_rv = ppi_callOriginal(NtOpenKeyEx, &KeyHandle, KEY_READ, &ObjectAttributes, req->OpenOptions);

	HANDLE peerFileHandle;

	if (DuplicateHandle(GetCurrentProcess(), KeyHandle, hPeerProcess, &response->KeyHandle, NULL, true, DUPLICATE_SAME_ACCESS|DUPLICATE_CLOSE_SOURCE) != 0) {
		//PPI_INFO("[+] Duplicate succeed: handle in peer: %Xh", response->KeyHandle);
	} else {
		PPI_CRITICAL("Failed to duplicate handle");
		response->l_rv = STATUS_ACCESS_DENIED;
	}

	/* FileHandle is closed by DuplicateHandle */
out:
	ppi_pkt_unpackObjectAttributesCleanUp(ObjectAttributes);
	return;
	
deny_open:
	response->l_rv = STATUS_ACCESS_DENIED;
	goto out;
}