//
// Portable Integrity Protection (PIP) System -
// Copyright (C) 2015 Secure Systems Laboratory, Stony Brook University
//
// This file is part of Portable Integrity Protection (PIP) System.
//
// Portable Integrity Protection (PIP) System is free software: you can
// redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Portable Integrity Protection (PIP) System is distributed in the hope that it
// will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Portable Integrity Protection (PIP) System.  If not, see
// <http://www.gnu.org/licenses/>.
//

#include "ppi_config.h"
#include "ppi_helperConnection.h"
#include "ppi_calls.h"
#include "ppi_debug.h"
#include "ppi_utils.h"
#include "ppi_level.h"
#include "ppi_bufferManager.h"

#include "ppi_redirect.h"

#include "ppi_regHandleManager.h"

using namespace std;

struct hPath {
	HANDLE root;
	void *path;
};


ppi_winapi_prologue(NtSetValueKey, NTSTATUS, 6,
  _In_      HANDLE, KeyHandle,
  _In_      PUNICODE_STRING, ValueName,
  _In_opt_  ULONG, TitleIndex,
  _In_      ULONG, Type,
  _In_opt_  PVOID, Data,
  _In_      ULONG, DataSize
)

	NTSTATUS result;
	result = ppi_callOriginal(NtSetValueKey, KeyHandle, ValueName, TitleIndex, Type, Data, DataSize);

	if (result == STATUS_SUCCESS) {
		PPI_INFO("Write to handle %x", KeyHandle);
	} else {
		PPI_INFO("Failed with error %x for handle %x", result, KeyHandle);		
	}
	
	ppi_regHandleManager_printHandlePath(KeyHandle);

ppi_winapi_epilogue(NtSetValueKey, result)



ppi_winapi_prologue(NtOpenKeyTransacted, NTSTATUS, 4,
  _Out_  PHANDLE, KeyHandle,
  _In_   ACCESS_MASK, DesiredAccess,
  _In_   POBJECT_ATTRIBUTES, ObjectAttributes,
  _In_   HANDLE, TransactionHandle
)
	NTSTATUS result;
	ppi_call2Hooked(NtOpenKeyTransactedEx, result, KeyHandle, DesiredAccess, ObjectAttributes, 0, TransactionHandle);

ppi_winapi_epilogue(NtOpenKeyTransacted, result)

ppi_winapi_prologue(NtOpenKeyTransactedEx, NTSTATUS, 5,
  _Out_  PHANDLE, KeyHandle,
  _In_   ACCESS_MASK, DesiredAccess,
  _In_   POBJECT_ATTRIBUTES, ObjectAttributes,
  _In_   ULONG, OpenOptions,
  _In_   HANDLE, TransactionHandle
)
	NTSTATUS result;
	result = ppi_callOriginal(NtOpenKeyTransactedEx, KeyHandle, DesiredAccess, ObjectAttributes, OpenOptions, TransactionHandle);
	/*
	if (result == 0) {
		wchar_t *regName = (wchar_t *)ppi_bm_malloc(PPI_PATH_WCHAR_SIZE*sizeof(wchar_t));
		ppi_utils_extractFilePathFromOA(ObjectAttributes, regName, PPI_PATH_WCHAR_SIZE*sizeof(wchar_t));
		if (DesiredAccess & KEY_WRITE) {
			//PPI_INFO("Write access to regName %ls gave handle %x, root handle: %x", regName, *KeyHandle, ObjectAttributes->RootDirectory);
		} else {
			//PPI_INFO("Other access to regName %ls gave handle %x, root handle: %x", regName, *KeyHandle, ObjectAttributes->RootDirectory);
		}
		ppi_bm_free(regName);
	}
	*/
ppi_winapi_epilogue(NtOpenKeyTransactedEx, result)



ppi_winapi_prologue(NtOpenKeyEx, NTSTATUS, 4,
  _Out_       PHANDLE, KeyHandle,
  _In_        ACCESS_MASK, DesiredAccess,
  _In_        POBJECT_ATTRIBUTES, ObjectAttributes,
  _In_        ULONG, OpenOptions
)
	NTSTATUS result;

	POBJECT_ATTRIBUTES oa = NULL, original_oa = ObjectAttributes;

	PPI_RegOA ppi_oa = PPI_RegOA(ObjectAttributes);
	PPI_CRITICAL("Path of REG is %ls", ppi_oa.getPrintablePath());

	/*
	oa = ppi_convertRegObjectAttributes2Benign(ObjectAttributes);

	if (oa) {
		result = ppi_callOriginal(NtOpenKeyEx, KeyHandle, KEY_READ, oa, OpenOptions);

		if (result == STATUS_ACCESS_DENIED) {
			ObjectAttributes = oa;
			goto ask_helper_to_open;
		}
	}
	*/

	/* If cannot open benign, open the original */
	result = ppi_callOriginal(NtOpenKeyEx, KeyHandle, DesiredAccess, ObjectAttributes, OpenOptions);
	/*
	wchar_t *pathbuf = ppi_bm_path_malloc();
	GetNtPathFromHandle(*KeyHandle, pathbuf, PPI_PATH_WCHAR_SIZE);
	PPI_NEWAPI_LOG("Result: %d, Reg key is %ls", result, pathbuf);
	ppi_bm_free(pathbuf);
	*/


	if (result == STATUS_ACCESS_DENIED) {
ask_helper_to_open:
		ppi_pkt_prepare_packets(NtOpenKeyEx, pkt, response);
		pkt.OpenOptions = OpenOptions;
		pkt.DesiredAccess = DesiredAccess;
		ppi_pkt_packObjectAttributes(ObjectAttributes, pkt);
			
		if (ppi_helper_sendRecvCorrectly(pkt, response)) {
			result = response.l_rv;
			if (response.l_rv == STATUS_SUCCESS) {
				*KeyHandle = response.KeyHandle;
				PPI_INFO("Handle received from helper successfully: %x", *KeyHandle);
			} else {
				PPI_INFO("Helper failed to help");
			}
		}
	}
	
	if (result == STATUS_SUCCESS) {
		
		/*wchar_t *regName = (wchar_t *)ppi_bm_malloc(PPI_PATH_WCHAR_SIZE*sizeof(wchar_t));
		ppi_utils_extractFilePathFromOA(ObjectAttributes, regName, PPI_PATH_WCHAR_SIZE*sizeof(wchar_t));		
		ppi_regHandleManager_saveHandle(*KeyHandle, ObjectAttributes->RootDirectory, regName);*/
		
		ppi_regHandleManager_saveHandle(*KeyHandle, ObjectAttributes);
		ppi_regHandleManager_printHandlePath(*KeyHandle);
	}

	if (oa != NULL)
		ppi_deallocObjectAttributes(oa);
out:
ppi_winapi_epilogue(NtOpenKeyEx, result)


ppi_winapi_prologue(NtOpenKey, NTSTATUS, 3,
  _Out_       PHANDLE, KeyHandle,
  _In_        ACCESS_MASK, DesiredAccess,
  _In_        POBJECT_ATTRIBUTES, ObjectAttributes
)

	NTSTATUS result;
	ppi_call2Hooked(NtOpenKeyEx, result, KeyHandle, DesiredAccess, ObjectAttributes, 0);

ppi_winapi_epilogue(NtOpenKey, result)


ppi_winapi_prologue(NtCreateKey, NTSTATUS, 7,
  _Out_       PHANDLE, KeyHandle,
  _In_        ACCESS_MASK, DesiredAccess,
  _In_        POBJECT_ATTRIBUTES, ObjectAttributes,
  /*_Reserved_*/  ULONG, TitleIndex,
  _In_opt_    PUNICODE_STRING, Class,
  _In_        ULONG, CreateOptions,
  _Out_opt_   PULONG, Disposition
)

	NTSTATUS result;
	result = ppi_callOriginal(NtCreateKey, KeyHandle, DesiredAccess, ObjectAttributes, TitleIndex, Class, CreateOptions, Disposition);
	
	if (result != STATUS_SUCCESS) {
		wchar_t *regName = ppi_bm_path_malloc();
		ppi_utils_extractFilePathFromOA(ObjectAttributes, regName, PPI_PATH_WCHAR_SIZE*sizeof(wchar_t));
		PPI_INFO("Failed Access to regName %ls gave %x, handle %x due to error %x", regName, result, *KeyHandle, result);
		
		if (DesiredAccess & KEY_WRITE) {
			PPI_INFO("\tWrite access to regName %ls gave handle %x, root handle: %x", regName, *KeyHandle, ObjectAttributes->RootDirectory);
		} else {
			PPI_INFO("\tOther access to regName %ls gave handle %x, root handle: %x, access: %x", regName, *KeyHandle, ObjectAttributes->RootDirectory, DesiredAccess);
		}
		
		ppi_bm_free(regName);
		PPI_INFO("\t*********Root directory: ");
		ppi_regHandleManager_printHandlePath(ObjectAttributes->RootDirectory);

	} else {
		ppi_regHandleManager_saveHandle(*KeyHandle, ObjectAttributes);
		/*wchar_t *regName = ppi_bm_path_malloc();
		ppi_utils_extractFilePathFromOA(ObjectAttributes, regName, PPI_PATH_WCHAR_SIZE*sizeof(wchar_t));		
		ppi_regHandleManager_saveHandle(*KeyHandle, ObjectAttributes->RootDirectory, regName);*/
	}

ppi_winapi_epilogue(NtCreateKey, result)




ppi_winapi_prologue(NtOpenKeyNew, NTSTATUS, 3,
  _Out_       PHANDLE, KeyHandle,
  _In_        ACCESS_MASK, DesiredAccess,
  _In_        POBJECT_ATTRIBUTES, ObjectAttributes
)

	NTSTATUS result;

	PPI_RegOA ppi_oa = PPI_RegOA(ObjectAttributes);
	
	result = ppi_callOriginal(NtOpenKey, KeyHandle, DesiredAccess, ObjectAttributes);
	
	if (result != STATUS_SUCCESS) {
		ppi_pkt_prepare_packets(NtOpenKeyEx, pkt, response);
		pkt.OpenOptions = 0;
		pkt.DesiredAccess = DesiredAccess;

		if (ppi_oa.getNextTrustedOA() != NULL)
			ObjectAttributes = ppi_oa.getNextTrustedOA();

		ppi_pkt_packObjectAttributes(ObjectAttributes, pkt);
			
		if (ppi_helper_sendRecvCorrectly(pkt, response)) {
			NTSTATUS oldresult = result;
			result = response.l_rv;
			if (response.l_rv == STATUS_SUCCESS) {
				*KeyHandle = response.KeyHandle;
				PPI_INFO("Handle received from helper successfully: %x", *KeyHandle);
			} else {
				PPI_CRITICAL("Helper failed to help with error code %x", result);
				PPI_INFO("Helper failed to help");
				result = oldresult;
			}
		}
	}

ppi_winapi_epilogue(NtOpenKeyNew, result)
