//
// Portable Integrity Protection (PIP) System -
// Copyright (C) 2015 Secure Systems Laboratory, Stony Brook University
//
// This file is part of Portable Integrity Protection (PIP) System.
//
// Portable Integrity Protection (PIP) System is free software: you can
// redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Portable Integrity Protection (PIP) System is distributed in the hope that it
// will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Portable Integrity Protection (PIP) System.  If not, see
// <http://www.gnu.org/licenses/>.
//

/*
Sequence for including files so that won't complain for PUNICODE_STRING
http://social.msdn.microsoft.com/Forums/vstudio/en-US/6d8f8b75-c6aa-48a1-bad4-5c140f521fa7/compilation-error-after-including-ntsecpkgh-file
*/
//#include <windows.h>
//#include <Sspi.h>
//#include <NTSecAPI.h>
//#include <NTSecPkg.h>

#include "ppi_config.h"
#include "ppi_helperConnection.h"
#include "ppi_calls.h"
#include "ppi_debug.h"
#include "ppi_utils.h"
#include "ppi_level.h"
#include "ppi_redirect.h"
#include "ppi_bufferManager.h"

#include "ppi_principal.h"
#include "ppi_regHandleManager.h"


ppi_winapi_prologue(NtOpenFileNew, NTSTATUS, 6,
  _Out_  PHANDLE, FileHandle,
  _In_   ACCESS_MASK, DesiredAccess,
  _In_   POBJECT_ATTRIBUTES, ObjectAttributes,
  _Out_  PIO_STATUS_BLOCK, IoStatusBlock,
  _In_   ULONG, ShareAccess,
  _In_   ULONG, OpenOptions
)

	NTSTATUS result;
	ppi_call2Hooked(NtCreateFileNew, result, FileHandle, DesiredAccess, ObjectAttributes, IoStatusBlock, 0, FILE_ATTRIBUTE_NORMAL, ShareAccess, FILE_OPEN, OpenOptions, NULL, 0);

ppi_winapi_epilogue(NtOpenFileNew, result)



ppi_winapi_prologue(NtCreateFileNew, NTSTATUS, 11,
  _Out_     PHANDLE, FileHandle,
  _In_      ACCESS_MASK, DesiredAccess,
  _In_      POBJECT_ATTRIBUTES, ObjectAttributes,
  _Out_     PIO_STATUS_BLOCK, IoStatusBlock,
  _In_opt_  PLARGE_INTEGER, AllocationSize,
  _In_      ULONG, FileAttributes,
  _In_      ULONG, ShareAccess,
  _In_      ULONG, CreateDisposition,
  _In_      ULONG, CreateOptions,
  _In_      PVOID, EaBuffer,
  _In_      ULONG, EaLength
)

	NTSTATUS result;
	POBJECT_ATTRIBUTES oa = NULL;
	
	if (!(ObjectAttributes && ObjectAttributes->ObjectName && ObjectAttributes->ObjectName->Buffer))
		goto call_original_and_return;

	PPI_OA ppi_oa = PPI_OA(ObjectAttributes);


#define MODE_IS_READ ((CreateOptions & FILE_NON_DIRECTORY_FILE) && ((DesiredAccess & GENERIC_READ) || (DesiredAccess & FILE_READ_DATA)))
#define MODE_IS_WRITEFILE ((CreateOptions & FILE_NON_DIRECTORY_FILE) && ((DesiredAccess & GENERIC_WRITE) || (DesiredAccess & FILE_WRITE_DATA) || (DesiredAccess & FILE_APPEND_DATA)))


	if (ppi_oa.isRedirectable()) {

		switch (CreateDisposition) {

		case FILE_SUPERSEDE:
		case FILE_CREATE: {
				if (CreateDisposition == FILE_CREATE && ppi_oa.getFirstReadableOA() != NULL)
				ObjectAttributes = ppi_oa.getFirstReadableOA(); //Would fail because of file exists
				else {
					ObjectAttributes = ppi_oa.getPrivateOA();
				}
				break;
		}
		case FILE_OPEN:
		case FILE_OPEN_IF: {
				if (ppi_oa.getFirstReadableOA() != NULL) {
					// Copy file to private path if open in write mode
					if (MODE_IS_WRITEFILE) {
						ppi_oa.copyFile2private();
						ObjectAttributes = ppi_oa.getPrivateOA();
					}
					else {
						ObjectAttributes = ppi_oa.getFirstReadableOA();
					}
				}
				else {
					if (CreateDisposition == FILE_OPEN_IF) // Would fail for FILE_OPEN if file does not exist, no need to do anything
						ObjectAttributes = ppi_oa.getPrivateOA();

				}
				break;
		}

		case FILE_OVERWRITE:
		case FILE_OVERWRITE_IF: {
				if (ppi_oa.getFirstReadableOA() != NULL) {
					ppi_oa.copyFile2private();
					ObjectAttributes = ppi_oa.getPrivateOA();
				}
				else {
					if (CreateDisposition == FILE_OVERWRITE_IF)
						ObjectAttributes = ppi_oa.getPrivateOA();
				}
				break;
		}
		}
	}
	
	//////Check if file is untrusted requires READ_CONTROL??? What if failed to READ_CONTROL??
	result = ppi_callOriginal(NtCreateFile, FileHandle, DesiredAccess|READ_CONTROL, ObjectAttributes, IoStatusBlock, AllocationSize,
					FileAttributes, ShareAccess, CreateDisposition, CreateOptions, EaBuffer, EaLength);


	//Only non-redirectable contents can be untrusted
	if (!ppi_oa.isRedirectable() && MODE_IS_READ && result == STATUS_SUCCESS) {
checkHandleIsUntrusted:
		//Check if the file opened is untrusted or not
		PPI_OA actualOA = PPI_OA(ObjectAttributes);
		PPI_Resource ppi_resource = PPI_Resource(*FileHandle, &actualOA);
		if (!ppi_resource.isTrustedByPrincipal()) {
			ppi_callOriginal(NtClose, *FileHandle);
			result = STATUS_ACCESS_DENIED; //File is data, should be denied directly
			PPI_HIGH_VIOLATION("Opening untrusted file for reading data: %ls", actualOA.getPrintablePath());
		}

	} else if (!PPI_PRINCIPAL_IS_BENIGN() && (result == STATUS_ACCESS_DENIED || result == STATUS_OBJECT_NAME_INVALID)) {
		// Need to contact helper to obtain handle, or ask helper to perform the request
		// Because file is not accessible, or has no permission to create file
		ppi_pkt_prepare_packets(NtCreateFile, pkt, response);			
		pkt.DesiredAccess = DesiredAccess;
		memcpy(&(pkt.AllocationSize), &AllocationSize, sizeof(AllocationSize));
		pkt.FileAttributes = FileAttributes;
		pkt.ShareAccess = ShareAccess;
		pkt.CreateDisposition = CreateDisposition;
		pkt.CreateOptions = CreateOptions;
		pkt.EaLength = EaLength;
		pkt.EaBuffer_isNull = (EaBuffer == NULL);
		ppi_pkt_packObjectAttributes(ObjectAttributes, pkt);
			
		if (ppi_helper_sendRecvCorrectly(pkt, response)) {
			result = response.l_rv;
			*IoStatusBlock = response.IoStatusBlock;
			if (response.l_rv == STATUS_SUCCESS) {
				*FileHandle = response.FileHandle;
				goto checkHandleIsUntrusted;
			} else if (response.l_rv == STATUS_ACCESS_DENIED) {
				PPI_CRITICAL("Helper also failed to open the file %ls with permission error. Will create a shadow copy and retry", ppi_oa.getPrintablePath());
			} else {
				PPI_CRITICAL("Helper Failed with unknown error: status code: %Xh", response.l_rv);
			}
		} else {
			PPI_ERROR("Failed to send and receive from helper correctly");
		}
	}


	/*
		CreateDisposition value	| Action if file exists				| Action if file does not exist	| What to do?
		------------------------+-----------------------------------+-------------------------------+----------------------------------
		FILE_SUPERSEDE			| Replace the file.                 | Create the file.				| Always create in the private location
		FILE_CREATE				| Return an error.					| Create the file.				| Error if a copy exists, otherwise, creates in private
		FILE_OPEN				| Open the file.					| Return an error.				| Read-only: open a any copy; Write: copy to private before opening
								| RO: any copy, W: copy to private  |								|
		FILE_OPEN_IF			| Open the file.					| Create the file.				|
								| RO: any copy, W: copy to private	| Create in the private location|
		FILE_OVERWRITE			| Open the file, and overwrite it.  | Return an error.				|
								| Copy to private					|								|
		FILE_OVERWRITE_IF		| Open the file, and overwrite it.  | Create the file.				|
								| Copy to private					| 

		CreateDisposition value	| Action if file exists				| Action if file does not exist	| 
		------------------------+-----------------------------------+-------------------------------+
		FILE_SUPERSEDE			| Always create in the private location                             |
		FILE_CREATE				| Return an error.					| Create in private				|
		FILE_OPEN				| RO: any copy, W: copy to private  | Return an error				|
		FILE_OPEN_IF			| RO: any copy, W: copy to private	| Create in private				|
		FILE_OVERWRITE			| Copy to private					| Return an error				|
		FILE_OVERWRITE_IF		| Copy to private					| Create in private				|
	*/

	out:
ppi_winapi_epilogue(NtCreateFileNew, result)
