//
// Portable Integrity Protection (PIP) System -
// Copyright (C) 2015 Secure Systems Laboratory, Stony Brook University
//
// This file is part of Portable Integrity Protection (PIP) System.
//
// Portable Integrity Protection (PIP) System is free software: you can
// redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Portable Integrity Protection (PIP) System is distributed in the hope that it
// will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Portable Integrity Protection (PIP) System.  If not, see
// <http://www.gnu.org/licenses/>.
//

#include "ppi_config.h"
#include "ppi_helperConnection.h"
#include "ppi_calls.h"
#include "ppi_debug.h"
#include "ppi_utils.h"
#include "ppi_level.h"
#include "ppi_redirect.h"
#include "ppi_bufferManager.h"

ppi_winapi_prologue(NtQueryAttributesFile, NTSTATUS, 2,
  _In_   POBJECT_ATTRIBUTES, ObjectAttributes,
  _Out_  PFILE_BASIC_INFORMATION, FileInformation
)
	NTSTATUS result;

	PPI_OA ppi_oa = PPI_OA(ObjectAttributes);
	if (ppi_oa.isRedirectable() && ppi_oa.getFirstReadableOA() != NULL) {
			ObjectAttributes = ppi_oa.getFirstReadableOA();
	}

	result = ppi_callOriginal(NtQueryAttributesFile, ObjectAttributes, FileInformation);
	
	if (result == STATUS_ACCESS_DENIED) {

		ppi_pkt_prepare_packets(NtQueryAttributesFile, pkt, response);
		ppi_pkt_packObjectAttributes(ObjectAttributes, pkt);
		
		if (ppi_helper_sendRecvCorrectly(pkt, response)) {
			result = response.l_rv;
			*FileInformation = response.FileInformation;
		}
	}
	//XXX: Fine to just query attribute information on benign instead of the actual shadowed copy, if one exists.
	
out:
ppi_winapi_epilogue(NtQueryAttributesFile, result)



#ifdef WIN81

ppi_winapi_prologue(NtQueryFullAttributesFile, NTSTATUS, 2,
  _In_   POBJECT_ATTRIBUTES, ObjectAttributes,
  _Out_  PFILE_NETWORK_OPEN_INFORMATION, FileInformation
)

	NTSTATUS result;

	PPI_OA ppi_oa = PPI_OA(ObjectAttributes);
	if (ppi_oa.isRedirectable() && ppi_oa.getFirstReadableOA() != NULL) {
			ObjectAttributes = ppi_oa.getFirstReadableOA();
	}
	
	result = ppi_callOriginal(NtQueryFullAttributesFile, ObjectAttributes, FileInformation);
	
	if (result == STATUS_ACCESS_DENIED) {

		ppi_pkt_prepare_packets(NtQueryFullAttributesFile, pkt, response);
		ppi_pkt_packObjectAttributes(ObjectAttributes, pkt);
		
		if (ppi_helper_sendRecvCorrectly(pkt, response)) {
			result = response.l_rv;
			*FileInformation = response.FileInformation;
		}
	}
ppi_winapi_epilogue(NtQueryFullAttributesFile, result)

#endif


