/* * Portable Integrity Protection (PIP) System -
 * Copyright (C) 2015 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Portable Integrity Protection (PIP) System.
 *
 * Portable Integrity Protection (PIP) System is free software: you can
 * redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Portable Integrity Protection (PIP) System is distributed in the hope that it
 * will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Portable Integrity Protection (PIP) System.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#ifndef __PPI_REG_HANDLE_MANAGER_H__
#define __PPI_REG_HANDLE_MANAGER_H__

#include <windows.h>
#include <unordered_map>
#include "ppi_redirect.h"


void ppi_regHandleManager_handleClosed(HANDLE regHandle);
void ppi_regHandleManager_saveHandle(HANDLE regHandle, const OBJECT_ATTRIBUTES* oa);

void ppi_regHandleManager_printHandlePath(HANDLE regHandle);
DWORD GetNtPathFromHandle(HANDLE h_File, wchar_t* ps_NTPath, size_t size, size_t &copied);
    



class PPI_FileHandleManager {
public:
	static PPI_FileHandleManager &getInstance() {
		static PPI_FileHandleManager instance;
		static bool initialized = false;
		if (!initialized) {
			instance.initialize();
			initialized = true;
		}
		return instance;
	}
	~PPI_FileHandleManager() {
		for (unordered_map<HANDLE, wchar_t*>::iterator it = _map.begin(); it != _map.end(); it++)
			ppi_bm_free((*it).second);
	}
	void insertHandle(HANDLE h, const wchar_t *path);
	const wchar_t *getPathByHandle(HANDLE h);
	void closeHandle(HANDLE h);

private:
	PPI_FileHandleManager() {};
	PPI_FileHandleManager(PPI_PRINCIPAL_MANAGER const &);
	void operator=(PPI_FileHandleManager const &);
	void initialize() {_bufferCount = 0; _maxCount = 0;};
	unordered_map<HANDLE, wchar_t*> _map;
	int _bufferCount;
	int _maxCount;

};
void ppi_closeFileHandle(HANDLE FileHandle);



class PPI_RegOA {
public:
	const OBJECT_ATTRIBUTES *original_oa;

	PPI_RegOA(OBJECT_ATTRIBUTES *oa, const PPI_PRINCIPAL *principal=PPI_PRINCIPAL_MANAGER::getInstance().getCurrentPrincipal()): original_oa(oa), principal_(principal){
	
		this->printablePath_ = (wchar_t *)ppi_bm_malloc(PPI_PATH_WCHAR_SIZE*sizeof(wchar_t));
		if (this->printablePath_) {
			size_t copied = 0;
			if (oa->RootDirectory != NULL) {
				GetNtPathFromHandle(oa->RootDirectory, this->printablePath_, PPI_PATH_WCHAR_SIZE, copied);
			}
			this->printablePath_[copied] = L'\\';
			ppi_utils_extractFilePathFromOA(oa, (this->printablePath_+copied+1), PPI_PATH_WCHAR_SIZE*sizeof(wchar_t)-copied-1);
		}
		firstReadableOA_ = NULL;
	};

	const wchar_t *getPrintablePath() const {
		return printablePath_;
	};

	bool isRedirectable() const;
	OBJECT_ATTRIBUTES *getNextTrustedOA();

	~PPI_RegOA() {
		if (printablePath_)
			ppi_bm_free(printablePath_);
		if (firstReadableOA_ != NULL) {
			ppi_deallocUnicodeShadowPath(firstReadableOA_->ObjectName);
			ppi_bm_free(firstReadableOA_);
		}

	};
private:
	wchar_t *printablePath_;
	const PPI_PRINCIPAL *principal_;
	OBJECT_ATTRIBUTES *firstReadableOA_;
};

#endif /* __PPI_REG_HANDLE_MANAGER_H__ */
