//
// Portable Integrity Protection (PIP) System -
// Copyright (C) 2015 Secure Systems Laboratory, Stony Brook University
//
// This file is part of Portable Integrity Protection (PIP) System.
//
// Portable Integrity Protection (PIP) System is free software: you can
// redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Portable Integrity Protection (PIP) System is distributed in the hope that it
// will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Portable Integrity Protection (PIP) System.  If not, see
// <http://www.gnu.org/licenses/>.
//

#include "windows.h"
#include "ppi_debug.h"


int ppi_windowsIntegrity_getProcessIntegrityLevel()
{
 HANDLE hToken;
 HANDLE hProcess;

 DWORD dwLengthNeeded;
 DWORD dwError = ERROR_SUCCESS;

 PTOKEN_MANDATORY_LABEL pTIL = NULL;
 DWORD dwIntegrityLevel;
 int level = 0;
 
 hProcess = GetCurrentProcess();
 if (OpenProcessToken(hProcess, TOKEN_QUERY, &hToken)) 
 {
  // Get the Integrity level.
  if (!GetTokenInformation(hToken, TokenIntegrityLevel, 
      NULL, 0, &dwLengthNeeded))
  {
   dwError = GetLastError();
   if (dwError == ERROR_INSUFFICIENT_BUFFER)
   {
    pTIL = (PTOKEN_MANDATORY_LABEL)LocalAlloc(0, 
         dwLengthNeeded);
    if (pTIL != NULL)
    {
     if (GetTokenInformation(hToken, TokenIntegrityLevel, 
         pTIL, dwLengthNeeded, &dwLengthNeeded))
     {
      dwIntegrityLevel = *GetSidSubAuthority(pTIL->Label.Sid, 
        (DWORD)(UCHAR)(*GetSidSubAuthorityCount(pTIL->Label.Sid)-1));
 
      if (dwIntegrityLevel == SECURITY_MANDATORY_LOW_RID)
      {
       // Low Integrity
		//PPI_INFO("Low Process");
      }
      else if (dwIntegrityLevel >= SECURITY_MANDATORY_MEDIUM_RID && 
           dwIntegrityLevel < SECURITY_MANDATORY_HIGH_RID)
      {
       // Medium Integrity
		//PPI_INFO("Medium Process");
      }
      else if (dwIntegrityLevel >= SECURITY_MANDATORY_HIGH_RID)
      {
       // High Integrity
		//PPI_INFO("High integrity Process");
      }
      else if (dwIntegrityLevel >= SECURITY_MANDATORY_SYSTEM_RID)
      {
       // System Integrity
		//PPI_INFO("System Integrity Process");
      }
	  level = dwIntegrityLevel;
     }
     LocalFree(pTIL);
    }
   }
  }
  ppi_callOriginal(NtClose, hToken);
 }
 return level; 
}